<!--
  - babel-c.sidl.xsl: creates sidl interfaces from C headers
  -
  - Output should be directed to the file, Package_Class.sidl 
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

<xsl:import href="common.xsl"/>
<xsl:import href="common-sidl.xsl"/>
<xsl:import href="common-user.xsl"/>

<xsl:output method="text"/>

<!--
  - global variables:
  -   package
  -   version
  -
  - templates:
  -   createPackage
  -   
  -->

<xsl:param name="param_package"/>
<xsl:variable name="version" select="'1.0'"/>

<!--
  -  /library node
  -->
<xsl:template match="/library">
  <xsl:variable name="package">
    <xsl:value-of select="$param_package"/>
    <xsl:if test="$param_package = ''">
      <xsl:value-of select="scope[1]/@name"/>
    </xsl:if>
  </xsl:variable>
  <xsl:variable name="filename">
    <xsl:value-of select="concat($package, '.sidl')"/>
  </xsl:variable>

  <xsl:call-template name="openFile">
    <xsl:with-param name="filename" select="$filename"/>
  </xsl:call-template>

  <xsl:apply-templates select="scope[1]">
    <xsl:with-param name="package" select="$package"/>
  </xsl:apply-templates>

  <xsl:call-template name="closeFile"/>
</xsl:template>


<!--
  - scope node
  -->
<xsl:template match="scope">
  <xsl:param name="package" select="@name"/>

  <!-- open the sidl package -->
  <xsl:value-of select="concat('package ', $package)"/>
  <xsl:value-of select="concat(' version ', $version, ' {', $nl, $nl)"/>

  <xsl:call-template name="createInterface"/>

  <!-- close the sidl package -->
  <xsl:value-of select="concat('}', $nl)"/>

</xsl:template>


<!--
  - createInterface <scope>>
  -->
  <xsl:template name="createInterface">
    <xsl:value-of select="concat('  interface ', @name, $nl)"/>
    <xsl:value-of select="concat('  {', $nl)"/>

    <xsl:for-each select="method">
      <xsl:if test="@ctor != 'yes'">
        <xsl:if test="@dtor != 'yes'">
          <xsl:variable name="name">
            <xsl:call-template name="method-id"/>
          </xsl:variable>
          <xsl:call-template name="function-decl">
            <xsl:with-param name="id" select="$name"/>
            <xsl:with-param name="ws" select="'    '"/>
          </xsl:call-template>
        </xsl:if>
      </xsl:if>
    </xsl:for-each>

    <xsl:value-of select="concat('  }', $nl, $nl)"/>
  </xsl:template>


<!--
  - method-id <method>
  -->
  <xsl:template name="method-id">
    <xsl:choose>
      <xsl:when test="@overloaded = 'yes'">
        <xsl:value-of select="@name"/>
        <xsl:if test="@abbrev">
          <xsl:text>[</xsl:text>
          <xsl:value-of select="@abbrev"/>
          <xsl:text>]</xsl:text>
        </xsl:if>
      </xsl:when>
      <xsl:otherwise>
        <xsl:choose>
          <xsl:when test="@abbrev">
            <xsl:value-of select="@abbrev"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="@name"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


</xsl:stylesheet>
