<!--
  - babel-cpp.impl.xsl: creates a Babel Impl file from a C++ class
  -
  - Output should be directed to the file, Package_Class_Impl.cc
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

<xsl:import href="common.xsl"/>
<xsl:import href="common-sidl.xsl"/>
<xsl:import href="common-cpp.xsl"/>

<xsl:output method="text"/>

<!--
  - global variables:
  -   package
  -   comp_name
  -   comp_fullname
  -
  - templates:
  -   
  -->

<xsl:param name="param_package"/>


<!--
 -  /library node
 -->
<xsl:template match="/library">
  <xsl:variable name="package">
    <xsl:value-of select="$param_package"/>
    <xsl:if test="$param_package = ''">
      <xsl:for-each select="scope[1]">
        <xsl:call-template name="name-ifnot-alias"/>
      </xsl:for-each>
    </xsl:if>
  </xsl:variable>
  <xsl:variable name="comp_name">
    <xsl:for-each select="scope[1]/structure[1]">
      <xsl:call-template name="name-ifnot-alias"/>
    </xsl:for-each>
  </xsl:variable>
  <xsl:variable name="comp_fullname">
    <xsl:value-of select="concat($package, '.', $comp_name)"/>
  </xsl:variable>

  <xsl:call-template name="openFile">
    <xsl:with-param name="comp_fullname" select="$comp_fullname"/>
  </xsl:call-template>

  <xsl:apply-templates select="scope">
    <xsl:with-param name="package" select="$package"/>
    <xsl:with-param name="comp_fullname" select="$comp_fullname"/>
  </xsl:apply-templates>

  <xsl:call-template name="closeFile">
    <xsl:with-param name="comp_fullname" select="$comp_fullname"/>
  </xsl:call-template>

</xsl:template>


<!--
  - scope node
  -->
<xsl:template match="scope">
  <xsl:param name="package" select="@name"/>
  <xsl:param name="comp_fullname">
    <xsl:value-of select="concat($package, '.')"/>
    <xsl:value-of select="structure[1]/@name"/>
  </xsl:param>

  <xsl:apply-templates select="structure">
    <xsl:with-param name="package" select="$package"/>
    <xsl:with-param name="comp_fullname" select="$comp_fullname"/>
  </xsl:apply-templates>

</xsl:template>


<!--
  - structure node
  -->
<xsl:template match="structure">
  <xsl:param name="package" select="../@name"/>
  <xsl:param name="comp_fullname">
    <xsl:value-of select="concat($package, '.')"/>
    <xsl:call-template name="name-ifnot-alias"/>
  </xsl:param>

  <xsl:value-of select="$nl"/>
  <xsl:text>  // DO-NOT-DELETE splicer.begin(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._ctor)', $nl)"/>
  <xsl:text>  // add construction details here</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>  delegate = new </xsl:text>
  <xsl:if test="../@kind = 'namespace'">
    <xsl:value-of select="concat('::', ../@name, '::')"/>
  </xsl:if>
  <xsl:value-of select="concat(@name, '();', $nl)"/>

  <xsl:text>  // DO-NOT-DELETE splicer.end(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._ctor)', $nl)"/>

  <xsl:value-of select="$nl"/>
  <xsl:text>  // DO-NOT-DELETE splicer.begin(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._dtor)', $nl)"/>
  <xsl:text>  // add destruction details here</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:value-of select="concat('  delete delegate;', $nl)"/>

  <xsl:text>  // DO-NOT-DELETE splicer.end(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._dtor)', $nl)"/>

  <xsl:for-each select="method">
    <xsl:variable name="method_name">
      <xsl:call-template name="method-id"/>
    </xsl:variable>
    <xsl:if test="@ctor != 'yes'">
    <xsl:if test="@dtor != 'yes'">

      <xsl:value-of select="$nl"/>
      <xsl:text>  // DO-NOT-DELETE splicer.begin(</xsl:text>
      <xsl:value-of select="concat($comp_fullname, '.', $method_name, ')')"/>
      <xsl:value-of select="$nl"/>
      <xsl:text>  // Insert the implementation here...</xsl:text>
      <xsl:value-of select="$nl"/>

      <xsl:call-template name="decl-construct-list"/>
      <xsl:call-template name="assign-stmt-list"/>
      <xsl:call-template name="call-statement">
        <xsl:with-param name="id">
          <xsl:if test="@kind != 'static'">
            <xsl:text>delegate-></xsl:text>
          </xsl:if>
          <xsl:value-of select="@name"/>
        </xsl:with-param>
      </xsl:call-template>

      <xsl:text>  // DO-NOT-DELETE splicer.end(</xsl:text>
      <xsl:value-of select="concat($comp_fullname, '.', $method_name, ')')"/>
      <xsl:value-of select="$nl"/>

    </xsl:if>
    </xsl:if>
  </xsl:for-each>

</xsl:template>


<!--
  - type-decl-stmt <arg>
  -->
<xsl:template name="type-decl-stmt">
  <xsl:variable name="ws" select="'  '"/>
  <xsl:variable name="lowercase_name">
    <xsl:call-template name="lower-case">
      <xsl:with-param name="symbol" select="@name"/>
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="ext">
    <xsl:for-each select="type[1]">
      <xsl:call-template name="type-conv-name-ext"/>
    </xsl:for-each>
  </xsl:variable>

  <xsl:if test="$ext != ''">

    <xsl:value-of select="$ws"/>
    <xsl:for-each select="type[1]">
      <xsl:call-template name="decl-type-spec"/>
    </xsl:for-each>

    <!-- use actual arg for Babel opaque (user-defined) types -->

    <xsl:if test="type/@kind = 'usertype'">
      <xsl:value-of select="concat(', intent(', @intent, ')')"/>
      <xsl:value-of select="concat(' :: ', $lowercase_name)"/>
      <xsl:text> !! FIXME, delete previous declaration</xsl:text>
      <xsl:value-of select="$nl"/>
    </xsl:if>

    <xsl:if test="type/@kind != 'usertype'">
      <xsl:value-of select="concat(' :: ', $lowercase_name, $ext, $nl)"/>
    </xsl:if>

  </xsl:if>

</xsl:template>


<!--
  - assign-stmt <arg>
  -->
<xsl:template name="assign-stmt">
  <xsl:variable name="ws" select="'  '"/>
  <xsl:variable name="lowercase_name">
    <xsl:call-template name="lower-case">
      <xsl:with-param name="symbol" select="@name"/>
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="ext">
    <xsl:for-each select="type[1]">
      <xsl:call-template name="type-conv-name-ext"/>
    </xsl:for-each>
  </xsl:variable>

  <xsl:if test="$ext != ''">
    <xsl:value-of select="$ws"/>
    <xsl:value-of select="concat(@name, $ext, ' = ', $lowercase_name)"/>
    <xsl:if test="type/@kind = 'farray'">
      <xsl:text>%d_data</xsl:text>
    </xsl:if>
    <xsl:value-of select="$nl"/>
  </xsl:if>

</xsl:template>


<!--
  - act-arg-spec-list <method>
  -->
<xsl:template name="act-arg-spec-list">
  <xsl:for-each select="arg">
    <xsl:variable name="ext">
      <xsl:for-each select="type[1]">
        <xsl:call-template name="type-conv-name-ext"/>
      </xsl:for-each>
    </xsl:variable>

    <xsl:call-template name="lower-case">
      <xsl:with-param name="symbol" select="@name"/>
    </xsl:call-template>
    <xsl:value-of select="$ext"/>
    <xsl:if test="position() != last()">
      <xsl:text>, </xsl:text>
    </xsl:if>

  </xsl:for-each>

</xsl:template>


<!--
  - type-conv-name-ext <arg/type>
  -->
<xsl:template name="type-conv-name-ext">
  <xsl:param name="depth" select="0"/>
  <xsl:choose>
    <!-- Fortran types -->
    <xsl:when test="@kind = 'farray'">
      <xsl:text>_l</xsl:text>
    </xsl:when>
  </xsl:choose>
</xsl:template>


<!--
  - decl-type-spec <arg>
  -->
<xsl:template name="decl-type-spec">
  <xsl:choose>
    <xsl:when test="@kind = 'flogic'">
      <xsl:text>logical</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'fint'">
      <xsl:text>integer</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'ffloat'">
      <xsl:text>real</xsl:text>
      <xsl:if test="@fkind = 'dbl'">
        <xsl:text>(kind(1.0D0))</xsl:text>
      </xsl:if>
    </xsl:when>
    <xsl:when test="@kind = 'fcmplx'">
      <xsl:text>complex</xsl:text>
      <xsl:if test="@fkind = 'dbl'">
        <xsl:text>(kind(1.0D0))</xsl:text>
      </xsl:if>
    </xsl:when>
    <xsl:when test="@kind = 'fchar'">
      <xsl:text>character (len=</xsl:text>
      <xsl:value-of select="@clen"/>
      <xsl:text>)</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'farray'">
      <xsl:for-each select="array/type[1]">
        <xsl:call-template name="decl-type-spec">
        </xsl:call-template>
      </xsl:for-each>
      <xsl:text>, pointer, dimension(</xsl:text>
      <xsl:for-each select="array/dimension">
	<xsl:text>:</xsl:text>
        <xsl:if test="position() != last()">
          <xsl:text>,</xsl:text>
        </xsl:if>
      </xsl:for-each>
      <xsl:text>)</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'usertype'">
      <xsl:value-of select="concat('type(', @usertype, ')')"/>
    </xsl:when>
    <xsl:when test="@kind = 'fptr'"/>
    <xsl:otherwise>
      <xsl:text>UNSUPPORTED</xsl:text>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!--
  - openFile: print file header information
  -->
<xsl:template name="openFile">
  <xsl:param name="comp_fullname"/>

  <xsl:value-of select="$nl"/>
  <xsl:text>// DO-NOT-DELETE splicer.begin(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._includes)', $nl)"/>
  <xsl:text>// Put additional includes or other arbitrary code </xsl:text>
  <xsl:text>here...</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>// DO-NOT-DELETE splicer.end(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._includes)', $nl)"/>

</xsl:template>


<!--
  - closeFile: finish up
  -->
<xsl:template name="closeFile">
  <xsl:param name="comp_fullname"/>

  <xsl:value-of select="$nl"/>
  <xsl:text>// DO-NOT-DELETE splicer.begin(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._misc)', $nl)"/>
  <xsl:text>// Put miscellaneous code here</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>// DO-NOT-DELETE splicer.end(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._misc)', $nl)"/>

</xsl:template>


</xsl:stylesheet>
