<!--
  - babel-cpp.impl.hh.xsl: creates a Babel Impl.hh file from a C++ class
  -
  - Output should be directed to the file, Package_Class_Impl.hh.cc
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

<xsl:import href="common.xsl"/>

<xsl:output method="text"/>

<!--
  - global variables:
  -   param_package
  -   param_header
  -
  - templates:
  -   
  -->

<xsl:param name="param_package"/>
<xsl:param name="param_header"/>

<!--
 -  /library node
 -->
<xsl:template match="/library">
  <xsl:variable name="package">
    <xsl:value-of select="$param_package"/>
    <xsl:if test="$param_package = ''">
      <xsl:for-each select="scope[1]">
        <xsl:call-template name="name-ifnot-alias"/>
      </xsl:for-each>
    </xsl:if>
  </xsl:variable>
  <xsl:variable name="header_file">
    <xsl:value-of select="$param_header"/>
    <xsl:if test="$param_header = ''">
      <xsl:value-of select="scope[1]/structure[1]/@name"/>
      <xsl:text>.h</xsl:text>
    </xsl:if>
  </xsl:variable>
  <xsl:variable name="comp_name">
    <xsl:for-each select="scope[1]/structure[1]">
      <xsl:call-template name="name-ifnot-alias"/>
    </xsl:for-each>
  </xsl:variable>
  <xsl:variable name="comp_fullname">
    <xsl:value-of select="concat($package, '.', $comp_name)"/>
  </xsl:variable>

  <xsl:call-template name="openFile">
    <xsl:with-param name="comp_fullname" select="$comp_fullname"/>
    <xsl:with-param name="header_file" select="$header_file"/>
  </xsl:call-template>

  <xsl:apply-templates select="scope">
    <xsl:with-param name="package" select="$package"/>
    <xsl:with-param name="comp_fullname" select="$comp_fullname"/>
  </xsl:apply-templates>

  <xsl:call-template name="closeFile">
    <xsl:with-param name="comp_fullname" select="$comp_fullname"/>
  </xsl:call-template>

</xsl:template>


<!--
  - scope node
  -->
<xsl:template match="scope">
  <xsl:param name="package" select="@name"/>
  <xsl:param name="comp_fullname">
    <xsl:value-of select="concat($package, '.')"/>
    <xsl:value-of select="structure[1]/@name"/>
  </xsl:param>

  <xsl:apply-templates select="structure">
    <xsl:with-param name="package" select="$package"/>
    <xsl:with-param name="comp_fullname" select="$comp_fullname"/>
  </xsl:apply-templates>

</xsl:template>


<!--
  - structure node
  -->
<xsl:template match="structure">
  <xsl:param name="package" select="../@name"/>
  <xsl:param name="comp_fullname">
    <xsl:value-of select="concat($package, '.')"/>
    <xsl:call-template name="name-ifnot-alias"/>
  </xsl:param>

  <xsl:value-of select="$nl"/>
  <xsl:text>  // DO-NOT-DELETE splicer.begin(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._inherits)', $nl)"/>
  <xsl:text>  // Put additional inheritance here...</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>  // DO-NOT-DELETE splicer.end(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._inherits)', $nl)"/>

  <xsl:value-of select="$nl"/>
  <xsl:text>    // DO-NOT-DELETE splicer.begin(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._implementation)')"/>
  <xsl:value-of select="$nl"/>
  <xsl:text>    // Put additional implementation details here...</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:value-of select="concat('    ::', ../@name, '::', @name)"/>
  <xsl:text>* delegate;</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>    // DO-NOT-DELETE splicer.end(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._implementation)')"/>
  <xsl:value-of select="$nl"/>

</xsl:template>


<!--
  - openFile: print file header information
  -->
<xsl:template name="openFile">
  <xsl:param name="comp_fullname"/>
  <xsl:param name="header_file"/>

  <xsl:text>// DO-NOT-DELETE splicer.begin(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._includes)', $nl)"/>
  <xsl:text>// Put additional includes or other arbitrary code </xsl:text>
  <xsl:text>here...</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>#include "</xsl:text>
  <xsl:value-of select="$header_file"/>
  <xsl:text>"</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>// DO-NOT-DELETE splicer.end(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._includes)', $nl)"/>

</xsl:template>


<!--
  - closeFile: finish up
  -->
<xsl:template name="closeFile">
  <xsl:param name="comp_fullname"/>

  <xsl:value-of select="$nl"/>
  <xsl:text>// DO-NOT-DELETE splicer.begin(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._misc)', $nl)"/>
  <xsl:text>// Put miscellaneous things here...</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>// DO-NOT-DELETE splicer.end(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._misc)', $nl)"/>

</xsl:template>


</xsl:stylesheet>
