<!--
  - cca-cpp.impl.xsl: creates a Babel Impl file from a C++ Class
  -
  - Output should be directed to the file, Package_Class_Impl.cc
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

<xsl:import href="common.xsl"/>
<xsl:import href="common-sidl.xsl"/>
<xsl:import href="common-cpp.xsl"/>

<xsl:output method="text"/>

<!--
  - global variables:
  -   comp_name
  -   comp_package
  -   comp_fullname
  -   interface_file
  -   impl_scope
  -
  - templates:
  -   
  -->

<xsl:variable name="comp_name" select="/componentImplementation/@name"/>
<xsl:variable name="comp_package"
              select="/componentImplementation/@package"/>
<xsl:variable name="comp_fullname">
  <xsl:value-of select="concat($comp_package, '.', $comp_name)"/>
</xsl:variable>

<xsl:variable name="interface_file">
  <xsl:for-each
       select="/componentImplementation/language/property[@name='impl-xml']">
    <xsl:value-of select="@value"/>
  </xsl:for-each>
</xsl:variable>

<xsl:variable name="impl_scope">
  <xsl:for-each
       select="/componentImplementation/language/property[@name='impl-xml']">
    <xsl:value-of select="@value"/>
  </xsl:for-each>
</xsl:variable>


<!--
 -  /componentImplementation node
 -->
<xsl:template match="/componentImplementation">
  <xsl:call-template name="openFile"/>

  <xsl:call-template name="splicer-includes"/>
  <xsl:call-template name="splicer-ctor-impl"/>
  <xsl:call-template name="splicer-dtor-impl"/>
  <xsl:call-template name="splicer-load-impl"/>

  <xsl:for-each select="language/ports/provides/MethodsBlock/Method">
    <xsl:call-template name="subroutine-subprogram"/>
  </xsl:for-each>

  <xsl:for-each select="language/ports[1]">
    <xsl:call-template name="setServices"/>
  </xsl:for-each>

  <xsl:call-template name="closeFile"/>
</xsl:template>


<!--
  - splicer-includes </componentImplementation>
  -->
<xsl:template name="splicer-includes">
  <xsl:text>// DO-NOT-DELETE splicer.begin(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._includes)', $nl)"/>
  <xsl:text>// Put additional includes or other arbitrary code </xsl:text>
  <xsl:text>here...</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>// DO-NOT-DELETE splicer.end(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._includes)', $nl)"/>
  <xsl:value-of select="$nl"/>
</xsl:template>


<!--
  - splicer-ctor-impl </componentImplementation>
  -->
<xsl:template name="splicer-ctor-impl">
  <xsl:value-of select="$nl"/>
  <xsl:text>  // DO-NOT-DELETE splicer.begin(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._ctor)', $nl)"/>
  <xsl:text>  // add construction details here</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>  delegate = new </xsl:text>
  <xsl:if test="../@kind = 'namespace'">
    <xsl:value-of select="concat('::', ../@name, '::')"/>
  </xsl:if>
  <xsl:value-of select="concat(@name, '();', $nl)"/>

  <xsl:text>  // DO-NOT-DELETE splicer.end(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._ctor)', $nl)"/>
  <xsl:value-of select="$nl"/>
</xsl:template>


<!--
  - splicer-dtor-impl </componentImplementation>
  -->
<xsl:template name="splicer-dtor-impl">
  <xsl:value-of select="$nl"/>
  <xsl:text>  // DO-NOT-DELETE splicer.begin(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._dtor)', $nl)"/>
  <xsl:text>  // add destruction details here</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:value-of select="concat('  delete delegate;', $nl)"/>

  <xsl:text>  // DO-NOT-DELETE splicer.end(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._dtor)', $nl)"/>
  <xsl:value-of select="$nl"/>
</xsl:template>


<!--
  - splicer-load-impl </componentImplementation>
  -->
<xsl:template name="splicer-load-impl">
  <xsl:value-of select="$nl"/>
  <xsl:text>  // DO-NOT-DELETE splicer.begin(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._load)', $nl)"/>
  <xsl:text>  // Insert-Code-Here {</xsl:text>
  <xsl:value-of select="$comp_fullname"/>
  <xsl:text>._load} (class initialization)</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>  // DO-NOT-DELETE splicer.end(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._load)', $nl)"/>
  <xsl:value-of select="$nl"/>
</xsl:template>


<!--
  - subroutine-subprogram <MethodsBlock/Method>
  -->
<xsl:template name="subroutine-subprogram">
  <xsl:variable name="method_impl" select="@impl"/>

  <xsl:value-of select="$nl"/>
  <xsl:text>  // DO-NOT-DELETE splicer.begin(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '.', @name, ')', $nl)"/>
  <xsl:text>  // insert implementation here</xsl:text>
  <xsl:value-of select="$nl"/>

  <xsl:for-each select
       ="document($interface_file)/library/scope/method[@name=$method_impl]">
    <xsl:call-template name="decl-construct-list"/>
  </xsl:for-each>

  <xsl:for-each select
       ="document($interface_file)/library/scope/method[@name=$method_impl]">
    <xsl:call-template name="assign-stmt-list"/>
  </xsl:for-each>

  <xsl:for-each select
       ="document($interface_file)/library/scope/method[@name=$method_impl]">
    <xsl:call-template name="call-statement">
      <xsl:with-param name="id">
        <xsl:if test="@kind != 'static'">
          <xsl:text>delegate-></xsl:text>
        </xsl:if>
        <xsl:value-of select="@name"/>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:for-each>

  <xsl:text>  // DO-NOT-DELETE splicer.end(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '.', @name, ')', $nl)"/>
  <xsl:value-of select="$nl"/>

</xsl:template>


<!--
  - type-decl-stmt <arg>
  -->
<xsl:template name="type-decl-stmt">
  <xsl:variable name="ws" select="'  '"/>
  <xsl:variable name="lowercase_name">
    <xsl:call-template name="lower-case">
      <xsl:with-param name="symbol" select="@name"/>
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="ext">
    <xsl:for-each select="type[1]">
      <xsl:call-template name="type-conv-name-ext"/>
    </xsl:for-each>
  </xsl:variable>

  <xsl:if test="$ext != ''">

    <xsl:value-of select="$ws"/>
    <xsl:for-each select="type[1]">
      <xsl:call-template name="decl-type-spec"/>
    </xsl:for-each>

    <!-- use actual arg for Babel opaque (user-defined) types -->

    <xsl:if test="type/@kind = 'usertype'">
      <xsl:value-of select="concat(', intent(', @intent, ')')"/>
      <xsl:value-of select="concat(' :: ', $lowercase_name)"/>
      <xsl:text> !! FIXME, delete previous declaration</xsl:text>
      <xsl:value-of select="$nl"/>
    </xsl:if>

    <xsl:if test="type/@kind != 'usertype'">
      <xsl:value-of select="concat(' :: ', $lowercase_name, $ext, $nl)"/>
    </xsl:if>

  </xsl:if>

</xsl:template>


<!--
  - assign-stmt <arg>
  -->
<xsl:template name="assign-stmt">
  <xsl:variable name="ws" select="'  '"/>
  <xsl:variable name="lowercase_name">
    <xsl:call-template name="lower-case">
      <xsl:with-param name="symbol" select="@name"/>
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="ext">
    <xsl:for-each select="type[1]">
      <xsl:call-template name="type-conv-name-ext"/>
    </xsl:for-each>
  </xsl:variable>

  <xsl:if test="$ext != ''">
    <xsl:value-of select="$ws"/>
    <xsl:value-of select="concat(@name, $ext, ' = ', $lowercase_name)"/>
    <xsl:if test="type/@kind = 'farray'">
      <xsl:text>%d_data</xsl:text>
    </xsl:if>
    <xsl:value-of select="$nl"/>
  </xsl:if>

</xsl:template>


<!--
  - act-arg-spec-list <method>
  -->
<xsl:template name="act-arg-spec-list">
  <xsl:for-each select="arg">
    <xsl:variable name="ext">
      <xsl:for-each select="type[1]">
        <xsl:call-template name="type-conv-name-ext"/>
      </xsl:for-each>
    </xsl:variable>

    <xsl:call-template name="lower-case">
      <xsl:with-param name="symbol" select="@name"/>
    </xsl:call-template>
    <xsl:value-of select="$ext"/>
    <xsl:if test="position() != last()">
      <xsl:text>, </xsl:text>
    </xsl:if>

  </xsl:for-each>

</xsl:template>


<!--
  - type-conv-name-ext <arg/type>
  -->
<xsl:template name="type-conv-name-ext">
  <xsl:param name="depth" select="0"/>
  <xsl:choose>
    <!-- Fortran types -->
    <xsl:when test="@kind = 'farray'">
      <xsl:text>_l</xsl:text>
    </xsl:when>
  </xsl:choose>
</xsl:template>


<!--
  - decl-type-spec <arg>
  -->
<xsl:template name="decl-type-spec">
  <xsl:choose>
    <xsl:when test="@kind = 'flogic'">
      <xsl:text>logical</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'fint'">
      <xsl:text>integer</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'ffloat'">
      <xsl:text>real</xsl:text>
      <xsl:if test="@fkind = 'dbl'">
        <xsl:text>(kind(1.0D0))</xsl:text>
      </xsl:if>
    </xsl:when>
    <xsl:when test="@kind = 'fcmplx'">
      <xsl:text>complex</xsl:text>
      <xsl:if test="@fkind = 'dbl'">
        <xsl:text>(kind(1.0D0))</xsl:text>
      </xsl:if>
    </xsl:when>
    <xsl:when test="@kind = 'fchar'">
      <xsl:text>character (len=</xsl:text>
      <xsl:value-of select="@clen"/>
      <xsl:text>)</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'farray'">
      <xsl:for-each select="array/type[1]">
        <xsl:call-template name="decl-type-spec">
        </xsl:call-template>
      </xsl:for-each>
      <xsl:text>, pointer, dimension(</xsl:text>
      <xsl:for-each select="array/dimension">
	<xsl:text>:</xsl:text>
        <xsl:if test="position() != last()">
          <xsl:text>,</xsl:text>
        </xsl:if>
      </xsl:for-each>
      <xsl:text>)</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'usertype'">
      <xsl:value-of select="concat('type(', @usertype, ')')"/>
    </xsl:when>
    <xsl:when test="@kind = 'fptr'"/>
    <xsl:otherwise>
      <xsl:text>UNSUPPORTED</xsl:text>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!--
  - setServices: write setServices splicer blocks <ports>
  -->
<xsl:template name="setServices">
  <xsl:value-of select="$nl"/>
  <xsl:text>  // DO-NOT-DELETE splicer.begin(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '.setServices)', $nl)"/>
  <xsl:text>  // insert implementation here</xsl:text>
  <xsl:value-of select="$nl"/>

  <xsl:text>  
    myServices = services;
    gov::cca::TypeMap tm = services.createTypeMap();
    if(tm._is_nil()) {
       fprintf(stderr, "Error:: %s:%d: gov::cca::TypeMap is nil\n",
          __FILE__, __LINE__);
       exit(1);  
    } 
    gov::cca::Port p = self;      //  Babel required casting
    if(p._is_nil()) {
       fprintf(stderr, "Error:: %s:%d: Error casting self to gov::cca::Port \n",
          __FILE__, __LINE__);
       exit(1);
    }</xsl:text>

  <xsl:value-of select="$nl"/>
    
  <xsl:apply-templates select="provides" mode="add-provides-port"/>
  <xsl:apply-templates select="uses" mode="register-uses-port"/>

  <xsl:text>
    
    gov::cca::ComponentRelease cr = self;  //  Babel required casting
    services.registerForRelease(cr);
    return;</xsl:text>  
  <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>

  <xsl:text>  // DO-NOT-DELETE splicer.end(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '.setServices)', $nl)"/>
  <xsl:value-of select="$nl"/>
</xsl:template>


<!--
  - add-provides-port <provides>
  -->
<xsl:template match="provides" mode="add-provides-port">
  <xsl:text>
    services.addProvidesPort(p,
                             "</xsl:text>                             
  <xsl:value-of select="concat(@name, '&quot;,')"/> <xsl:text>
                             "</xsl:text>                             
  <xsl:value-of select="concat(@package, '.', @name, '&quot;')"/>
  <xsl:text>, tm);</xsl:text>
  <xsl:value-of select="$nl"/>
</xsl:template>


<!--
  - register-uses-port <uses>
  -->
<xsl:template match="uses" mode="register-uses-port">
  <xsl:text>
    services.registerUsesPort("</xsl:text>                             
  <xsl:value-of select="concat(@name, '&quot;,')"/> <xsl:text>
                              "</xsl:text>                             
  <xsl:value-of select="concat(@package, '.', @name, '&quot;')"/>
  <xsl:text>, tm);</xsl:text>
  <xsl:value-of select="$nl"/>
</xsl:template>


<!--
  - openFile: print file header information
  -->
<xsl:template name="openFile"/>


<!--
  - closeFile: finish up
  -->
<xsl:template name="closeFile">
  <xsl:value-of select="$nl"/>
  <xsl:text>// DO-NOT-DELETE splicer.begin(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._misc)', $nl)"/>
  <xsl:text>// Put miscellaneous code here</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>// DO-NOT-DELETE splicer.end(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._misc)', $nl)"/>
</xsl:template>


</xsl:stylesheet>
