<!--
  - cca-f90.sidl.xsl: creates a sidl interface description from a Fortran
  -                   module
  -
  - Output should be directed to the file, Package_Port.sidl 
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

<xsl:import href="common.xsl"/>
<xsl:import href="common-sidl.xsl"/>

<xsl:output method="text"/>

<!--
  - global variables:
  -   port_name
  -   interface_file
  -   impl_scope
  -
  - templates:
  -   createPackage
  -   
  -->

<xsl:variable name="port_name" select="/Symbol/SymbolName/@name"/>
  <xsl:variable name="interface_file">
  <xsl:for-each select="/Symbol/Metadata/MetadataEntry[@key='impl-xml']">
    <xsl:value-of select="@value"/>
  </xsl:for-each>
</xsl:variable>
<xsl:variable name="impl_scope">
  <xsl:for-each select="/Symbol/Metadata/MetadataEntry[@key='impl-scope']">
    <xsl:value-of select="@value"/>
  </xsl:for-each>
</xsl:variable>


<!--
  -  /Symbol node
  -->
<xsl:template match="/Symbol">
  <xsl:variable name="filename">
    <xsl:value-of select="concat(Port[1]/@package, '_')"/>
    <xsl:value-of select="concat(Port[1]/@name, '.sidl')"/>
  </xsl:variable>
  <xsl:call-template name="openFile">
    <xsl:with-param name="filename" select="$filename"/>
  </xsl:call-template>

  <xsl:apply-templates select="Port"/>

  <xsl:call-template name="closeFile">
    <xsl:with-param name="filename" select="$filename"/>
  </xsl:call-template>
</xsl:template>


<!--
  - Port node
  -->
<xsl:template match="Port">
  <xsl:variable name="port_type" select="$port_name"/>

  <!-- open the sidl package -->
  <xsl:value-of select="concat('package ', @package, ' version 1.0 {', $nl)"/>
  <xsl:value-of select="$nl"/>

  <xsl:call-template name="createPackage"/>

  <!-- close the sidl package -->
  <xsl:value-of select="concat('}', $nl)"/>

</xsl:template>


<!--
  - createPackage <Port>
  -->
  <xsl:template name="createPackage">

    <xsl:value-of select="concat('  interface ', @name)"/>
    <xsl:value-of select="concat(' extends gov.cca.Port', $nl)"/>
    <xsl:value-of select="concat('  {', $nl)"/>
    <xsl:apply-templates select="MethodsBlock/Method" mode="declMethod"/>
    <xsl:value-of select="concat('  }', $nl, $nl)"/>

  </xsl:template>


  <!--
    - declMethod <MethodsBlock/Method node>
    -->
  <xsl:template match="MethodsBlock/Method" mode="declMethod">
    <xsl:apply-templates mode="declMethod"
         select="document($interface_file)/library/scope[@name=$impl_scope]">
      <xsl:with-param name="method_name" select="@name"/>
      <xsl:with-param name="method_impl" select="@impl"/>
    </xsl:apply-templates>
  </xsl:template>


<!--
  - declMethod (library/scope node)
  -->
<xsl:template match="library/scope" mode="declMethod">
  <xsl:param name="method_name"/>
  <xsl:param name="method_impl"/>

  <xsl:for-each select="method[@name=$method_impl]">
    <xsl:call-template name="function-decl">
      <xsl:with-param name="id" select="$method_name"/>
      <xsl:with-param name="ws" select="'    '"/>
    </xsl:call-template>
  </xsl:for-each>

  </xsl:template>


</xsl:stylesheet>
