<!--
  - cca-f90.sidl.xsl: creates a sidl interface description from a Fortran
  -                   module
  -
  - Output should be directed to the file, Package_Port.sidl 
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

<xsl:import href="common.xsl"/>
<xsl:import href="common-sidl.xsl"/>

<xsl:output method="text"/>

<!--
  - global variables:
  -   class_name
  -   class_package
  -
  - templates:
  -   createPackage
  -   
  -->

<xsl:variable name="class_name" select="/componentImplementation/@name"/>
<xsl:variable name="class_package" select="/componentImplementation/@package"/>


<!--
  -  /componentImplementation node
  -->
<xsl:template match="/componentImplementation">
  <xsl:variable name="filename">
    <xsl:value-of select="concat(@package, '_')"/>
    <xsl:value-of select="concat(@name, '.sidl')"/>
  </xsl:variable>
  <xsl:call-template name="openFile">
    <xsl:with-param name="filename" select="$filename"/>
  </xsl:call-template>

  <xsl:apply-templates select="language/ports"/>

  <xsl:call-template name="closeFile">
    <xsl:with-param name="filename" select="$filename"/>
  </xsl:call-template>
</xsl:template>


<!--
  - ports node
  -->
<xsl:template match="language/ports">
  <xsl:variable name="port_type" select="$class_name"/>

  <!-- open the sidl package -->
  <xsl:value-of select="concat('package ', $class_package, ' version 1.0 {')"/>
  <xsl:value-of select="$nl"/>

  <xsl:call-template name="createPackage"/>

</xsl:template>


<!--
  - createPackage <ports>
  -->
  <xsl:template name="createPackage">

    <xsl:value-of select="concat('  class ', $class_name)"/>
    <xsl:text> implements-all gov.cca.Component,</xsl:text>
    <xsl:value-of select="$nl"/>
    <xsl:text>                        gov.cca.ComponentRelease</xsl:text>
    <xsl:apply-templates select="provides" mode="decl-impl-port"/>
    <xsl:value-of select="concat($nl, '  {')"/>
    <xsl:value-of select="concat($nl, '  }', $nl)"/>
    <xsl:value-of select="concat('}', $nl)"/>

  </xsl:template>


  <!--
    - decl-impl-port <provides node>
    -->
  <xsl:template match="provides" mode="decl-impl-port">
    <xsl:text>,</xsl:text>  <xsl:value-of select="$nl"/>
    <xsl:text>                        </xsl:text>
    <xsl:value-of select="concat(@package, '.', @name)"/>
  </xsl:template>

</xsl:stylesheet>
