<!--
  - cca-f90.impl.xsl: creates a Babel _Impl.F90 file for a CCA component from a
  -                   Fortran module
  -
  - Output should be directed to the file, Package_Class_Impl.F90
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

<xsl:import href="common.xsl"/>
<xsl:import href="common-sidl.xsl"/>
<xsl:import href="common-f90.xsl"/>

<xsl:output method="text"/>

<!--
  - global variables:
  -   comp_name
  -   comp_package
  -   comp_fullname
  -   comp_typename
  -   interface_file
  -   impl_scope
  -
  - templates:
  -   
  -->

<xsl:variable name="comp_name" select="/componentImplementation/@name"/>
<xsl:variable name="comp_package"
              select="/componentImplementation/@package"/>
<xsl:variable name="comp_fullname">
  <xsl:value-of select="concat($comp_package, '.', $comp_name)"/>
</xsl:variable>
<xsl:variable name="comp_typename">
  <xsl:value-of select="concat($comp_package, '_', $comp_name)"/>
</xsl:variable>

<xsl:variable name="interface_file">
  <xsl:for-each
       select="/componentImplementation/language/property[@name='impl-xml']">
    <xsl:value-of select="@value"/>
  </xsl:for-each>
</xsl:variable>

<xsl:variable name="impl_scope">
  <xsl:for-each
       select="/componentImplementation/language/property[@name='impl-scope']">
    <xsl:value-of select="@value"/>
  </xsl:for-each>
</xsl:variable>


<!--
 -  /componentImplementation node
 -->
<xsl:template match="/componentImplementation">
  <xsl:call-template name="openFile"/>

  <xsl:call-template name="splicer-misc"/>

  <xsl:call-template name="splicer-ctor-impl"/>
  <xsl:call-template name="splicer-dtor-impl"/>

  <xsl:for-each select="language/ports/provides/MethodsBlock/Method">
    <xsl:call-template name="subroutine-subprogram"/>
  </xsl:for-each>

  <xsl:for-each select="language/ports[1]">
    <xsl:call-template name="setServices"/>
  </xsl:for-each>

  <xsl:call-template name="closeFile"/>
</xsl:template>


<!--
  - splicer-misc </componentImplementation>
  -->
<xsl:template name="splicer-misc">

<xsl:text>
! DO-NOT-DELETE splicer.begin(_miscellaneous_code_start)
! Header files with short name definitions
</xsl:text>
  <xsl:apply-templates select="language/ports/uses" mode="include-uses">
  </xsl:apply-templates>
<xsl:text>! DO-NOT-DELETE splicer.end(_miscellaneous_code_start)</xsl:text>

</xsl:template>


<!--
  - include-uses <ports/uses>
  -->
<xsl:template match="uses" mode="include-uses">

  <xsl:text>#include "</xsl:text>
  <xsl:value-of select="concat(@package, '_', @name, '_fAbbrev.h&quot;')"/>
  <xsl:value-of select="$nl"/>

</xsl:template>


<!--
  - splicer-ctor-impl </componentImplementation>
  -->
<xsl:template name="splicer-ctor-impl">

  <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>
  <xsl:text>  ! DO-NOT-DELETE splicer.begin(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._ctor)', $nl)"/>
  <xsl:text>  ! Insert the implementation here...</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>
  <xsl:text>  ! Access private data</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>  type(</xsl:text>
  <xsl:value-of select="concat(@package, '_', @name, '_wrap) :: dp', $nl)"/>
  <xsl:text>  ! Allocate memory and initialize</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>  allocate(dp%d_private_data)</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>  call set_null(dp%d_private_data%frameworkServices)</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>  call </xsl:text>
  <xsl:value-of select="concat(@package, '_', @name, '__set_data_m')"/>
  <xsl:text>(self, dp)</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>
  <xsl:text>  ! DO-NOT-DELETE splicer.end(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._ctor)', $nl)"/>

</xsl:template>


<!--
  - splicer-dtor-impl </componentImplementation>
  -->
<xsl:template name="splicer-dtor-impl">

  <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>
  <xsl:text>  ! DO-NOT-DELETE splicer.begin(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._dtor)', $nl)"/>
  <xsl:text>  ! Insert the implementation here...</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>
  <xsl:text>  ! Access private data and deallocate storage</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>  type(</xsl:text>
  <xsl:value-of select="concat(@package, '_', @name, '_wrap) :: dp', $nl)"/>
  <xsl:text>  call </xsl:text>
  <xsl:value-of select="concat(@package, '_', @name, '__get_data_m')"/>
  <xsl:text>(self, dp)</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>
  <xsl:text>  ! Decrement reference count for framework services handle
  if (not_null(dp%d_private_data%frameworkServices)) then
     call deleteRef(dp%d_private_data%frameworkServices)
  end if

  deallocate(dp%d_private_data)

  ! DO-NOT-DELETE splicer.end(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '._dtor)', $nl)"/>

</xsl:template>


<!--
  - subroutine-subprogram <MethodsBlock/Method>
  -->
<xsl:template name="subroutine-subprogram">
  <xsl:variable name="method_impl" select="@impl"/>

  <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>
  <xsl:text>  ! DO-NOT-DELETE splicer.begin(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '.', @name, '.use)', $nl)"/>
  <xsl:text>  ! Insert use statements here...</xsl:text>
  <xsl:value-of select="$nl"/>

  <xsl:apply-templates
       select="/componentImplementation/language/ports/provides"
       mode="use-provides"/>

  <xsl:apply-templates
       select="/componentImplementation/language/ports/uses"
       mode="use-uses"/>

  <xsl:text>  use gov_cca_Services</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>  use gov_cca_Port</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>  use sidl_BaseInterface</xsl:text> <xsl:value-of select="$nl"/>

  <xsl:text>  use </xsl:text>  <xsl:value-of select="$impl_scope"/>
  <xsl:value-of select="$nl"/>

  <xsl:for-each select
       ="document($interface_file)/library/scope/method[@name=$method_impl]">
    <xsl:call-template name="use-stmt-list"/>
  </xsl:for-each>

  <xsl:text>  ! DO-NOT-DELETE splicer.end(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '.', @name, '.use)', $nl)"/>
  <xsl:value-of select="$nl"/>

  <xsl:text>  ! DO-NOT-DELETE splicer.begin(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '.', @name, ')', $nl)"/>
  <xsl:text>  ! Insert the implementation here...</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>

  <xsl:text>  logical :: proceed</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>

  <xsl:apply-templates select="/componentImplementation/language/ports/uses"
                       mode="decl-uses-impl"/>

  <xsl:text>  type(SIDL_BaseInterface_t) :: excpt</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>

  <xsl:for-each select
       ="document($interface_file)/library/scope/method[@name=$method_impl]">
    <xsl:call-template name="decl-construct-list"/>
  </xsl:for-each>

  <xsl:for-each select
       ="document($interface_file)/library/scope/method[@name=$method_impl]">
    <xsl:call-template name="assign-stmt-list"/>
  </xsl:for-each>

  <xsl:text>  ! Private data reference</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:value-of select="concat('  type(', $comp_typename, '_wrap) :: dp')"/>
  <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>

  <xsl:text>  ! Access private data</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:value-of select="concat('  call ', $comp_typename, '__get_data_m')"/>
  <xsl:text>(self, dp)</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>

  <xsl:text>  proceed = .true.
  if (is_null(dp%d_private_data%frameworkServices)) then
    proceed = .false.
  end if

  ! Obtain handles to uses ports</xsl:text>

  <xsl:value-of select="$nl"/>

  <xsl:apply-templates select="/componentImplementation/language/ports/uses"
                       mode="call-getPort"/>

  <xsl:text>
  !
  ! Make call to legacy procedure
  !
  ! WARNING, call (and procedure) will have to change if there are uses ports
  !          The uses ports should be passed to the procedure and called there
  !          at the appropriate point
  !</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>

  <xsl:for-each select
       ="document($interface_file)/library/scope/method[@name=$method_impl]">
    <xsl:call-template name="call-stmt"/>
  </xsl:for-each>

  <xsl:text>
  !
  ! Cleanup (delete dangling references and release ports)
  !</xsl:text>
  <xsl:value-of select="$nl"/>

  <xsl:apply-templates select="/componentImplementation/language/ports/uses"
                       mode="call-releasePort"/>

  <xsl:text>  ! DO-NOT-DELETE splicer.end(</xsl:text>
  <xsl:value-of select="concat($comp_fullname, '.', @name, ')', $nl)"/>

</xsl:template>


<!--
  - use-uses <ports/provides>
  -->
<xsl:template match="provides" mode="use-provides">
  <xsl:text>  use </xsl:text>
  <xsl:value-of select="concat(@package, '_', @name)"/>
  <xsl:value-of select="$nl"/>
</xsl:template>


<!--
  - use-uses <ports/uses>
  -->
<xsl:template match="uses" mode="use-uses">
  <xsl:text>  use </xsl:text>
  <xsl:value-of select="concat(@package, '_', @name)"/>
  <xsl:value-of select="$nl"/>
</xsl:template>


<!--
  - decl-uses-impl <ports/uses>
  -->
<xsl:template match="uses" mode="decl-uses-impl">
  <xsl:text>  type(gov_cca_Port_t) :: base</xsl:text>
  <xsl:value-of select="concat(@name, $nl)"/>
  <xsl:text>  type(</xsl:text>
  <xsl:value-of select="concat(@package, '_', @name, '_t) :: ')"/>
  <xsl:value-of select="concat('my', @name)"/>
  <xsl:value-of select="$nl"/>
  <xsl:text>  type(SIDL_BaseInterface_t) :: excpt</xsl:text>
  <xsl:value-of select="@name"/>
  <xsl:value-of select="$nl"/>
</xsl:template>


<!--
  - call-getPort <ports/uses>
  -->
<xsl:template match="uses" mode="call-getPort">
  <xsl:text>
  if (proceed) then
     call getPort(dp%d_private_data%frameworkServices, &amp;
          "</xsl:text>
  <xsl:value-of select="concat(@name, '&quot;, base', @name)"/>
  <xsl:value-of select="concat(', excpt', @name, ')', $nl)"/>
  <xsl:text>    if (is_null(</xsl:text>
  <xsl:value-of select="concat('excpt', @name, ')) then', $nl)"/>
  <xsl:text>      call cast(</xsl:text>
  <xsl:value-of select="concat('base', @name, ', my', @name, ')')"/> <xsl:text>
      if (is_null(</xsl:text>
  <xsl:value-of select="concat('my', @name, ')) then')"/> <xsl:text>
        print *, "</xsl:text> <xsl:value-of select="$comp_fullname"/>
  <xsl:text> Exception: incompatible </xsl:text>
  <xsl:value-of select="@name"/> <xsl:text>"
        proceed = .false.
      end if
    else
      print *, "</xsl:text> <xsl:value-of select="$comp_fullname"/>
      <xsl:text> Exception: in call getPort('</xsl:text>
      <xsl:value-of select="@name"/> <xsl:text>')"
      proceed = .false.
    end if
  end if</xsl:text>
  <xsl:value-of select="$nl"/>  
</xsl:template>


<!--
  - call-releasePort <ports/uses>
  -->
<xsl:template match="uses" mode="call-releasePort">
  <xsl:text>
  if (is_null(</xsl:text>
  <xsl:value-of select="concat('excpt', @name, ')) then')"/> <xsl:text>
    call deleteRef(</xsl:text>
  <xsl:value-of select="concat('base', @name, ')')"/> <xsl:text>
  end if
  if (not_null(</xsl:text>
  <xsl:value-of select="concat('my', @name, ')) then')"/> <xsl:text>
    call releasePort(dp%d_private_data%frameworkServices, &amp;
         "</xsl:text>
  <xsl:value-of select="concat(@name, '&quot;, excpt)', $nl)"/>
  <xsl:call-template name="checkException">
    <xsl:with-param name="ws" select="'    '"/>
    <xsl:with-param name="msg">
    <xsl:text>in call releasePort('</xsl:text>
    <xsl:value-of select="@name"/> <xsl:text>')</xsl:text>
    </xsl:with-param>
  </xsl:call-template>
  <xsl:text>  end if</xsl:text>
  <xsl:value-of select="$nl"/>  
</xsl:template>


<!--
  - type-decl-stmt <arg>
  -->
<xsl:template name="type-decl-stmt">
  <xsl:variable name="ws" select="'  '"/>
  <xsl:variable name="lowercase_name">
    <xsl:call-template name="lower-case">
      <xsl:with-param name="symbol" select="@name"/>
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="ext">
    <xsl:for-each select="type[1]">
      <xsl:call-template name="type-conv-name-ext"/>
    </xsl:for-each>
  </xsl:variable>

  <xsl:if test="$ext != ''">

    <xsl:value-of select="$ws"/>
    <xsl:for-each select="type[1]">
      <xsl:call-template name="decl-type-spec"/>
    </xsl:for-each>

    <!-- use actual arg for Babel opaque (user-defined) types -->

    <xsl:if test="type/@kind = 'usertype'">
      <xsl:value-of select="concat(', intent(', @intent, ')')"/>
      <xsl:value-of select="concat(' :: ', $lowercase_name)"/>
      <xsl:text> !! FIXME, delete previous declaration</xsl:text>
      <xsl:value-of select="$nl"/>
    </xsl:if>

    <xsl:if test="type/@kind != 'usertype'">
      <xsl:value-of select="concat(' :: ', $lowercase_name, $ext, $nl)"/>
    </xsl:if>

  </xsl:if>

</xsl:template>


<!--
  - assign-stmt <arg>
  -->
<xsl:template name="assign-stmt">
  <xsl:variable name="ws" select="'  '"/>
  <xsl:variable name="lowercase_name">
    <xsl:call-template name="lower-case">
      <xsl:with-param name="symbol" select="@name"/>
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="ext">
    <xsl:for-each select="type[1]">
      <xsl:call-template name="type-conv-name-ext"/>
    </xsl:for-each>
  </xsl:variable>

  <xsl:if test="$ext != ''">
    <xsl:value-of select="$ws"/>
    <xsl:value-of select="concat(@name, $ext, ' = ', $lowercase_name)"/>
    <xsl:if test="type/@kind = 'farray'">
      <xsl:text>%d_data</xsl:text>
    </xsl:if>
    <xsl:value-of select="$nl"/>
  </xsl:if>

</xsl:template>


<!--
  - act-arg-spec-list <method>
  -->
<xsl:template name="act-arg-spec-list">
  <xsl:for-each select="arg">
    <xsl:variable name="ext">
      <xsl:for-each select="type[1]">
        <xsl:call-template name="type-conv-name-ext"/>
      </xsl:for-each>
    </xsl:variable>

    <xsl:call-template name="lower-case">
      <xsl:with-param name="symbol" select="@name"/>
    </xsl:call-template>
    <xsl:value-of select="$ext"/>
    <xsl:if test="position() != last()">
      <xsl:text>, </xsl:text>
    </xsl:if>

  </xsl:for-each>

</xsl:template>


<!--
  - type-conv-name-ext <arg/type>
  -->
<xsl:template name="type-conv-name-ext">
  <xsl:param name="depth" select="0"/>
  <xsl:choose>
    <!-- Fortran types -->
    <xsl:when test="@kind = 'farray'">
      <xsl:text>_l</xsl:text>
    </xsl:when>
  </xsl:choose>
</xsl:template>


<!--
  - decl-type-spec <arg>
  -->
<xsl:template name="decl-type-spec">
  <xsl:choose>
    <xsl:when test="@kind = 'flogic'">
      <xsl:text>logical</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'fint'">
      <xsl:text>integer</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'ffloat'">
      <xsl:text>real</xsl:text>
      <xsl:if test="@fkind = 'dbl'">
        <xsl:text>(kind(1.0D0))</xsl:text>
      </xsl:if>
    </xsl:when>
    <xsl:when test="@kind = 'fcmplx'">
      <xsl:text>complex</xsl:text>
      <xsl:if test="@fkind = 'dbl'">
        <xsl:text>(kind(1.0D0))</xsl:text>
      </xsl:if>
    </xsl:when>
    <xsl:when test="@kind = 'fchar'">
      <xsl:text>character (len=</xsl:text>
      <xsl:value-of select="@clen"/>
      <xsl:text>)</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'farray'">
      <xsl:for-each select="array/type[1]">
        <xsl:call-template name="decl-type-spec">
        </xsl:call-template>
      </xsl:for-each>
      <xsl:text>, pointer, dimension(</xsl:text>
      <xsl:for-each select="array/dimension">
	<xsl:text>:</xsl:text>
        <xsl:if test="position() != last()">
          <xsl:text>,</xsl:text>
        </xsl:if>
      </xsl:for-each>
      <xsl:text>)</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'usertype'">
      <xsl:value-of select="concat('type(', @usertype, ')')"/>
    </xsl:when>
    <xsl:when test="@kind = 'fptr'"/>
    <xsl:otherwise>
      <xsl:text>UNSUPPORTED</xsl:text>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!--
  - setServices: write setServices splicer blocks <ports>
  -->
<xsl:template name="setServices">

<xsl:text>

  ! DO-NOT-DELETE splicer.begin(</xsl:text>
     <xsl:value-of select="$comp_fullname"/> <xsl:text>.setServices.use)
  ! Insert use statements here...
  use gov_cca_TypeMap
  use gov_cca_Port
  ! DO-NOT-DELETE splicer.end(</xsl:text>
     <xsl:value-of select="$comp_fullname"/> <xsl:text>.setServices.use)

  ! DO-NOT-DELETE splicer.begin(</xsl:text>
     <xsl:value-of select="$comp_fullname"/> <xsl:text>.setServices)
  ! Insert the implementation here...</xsl:text>

  <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>

  <!-- declare character port names and port types -->

  <xsl:apply-templates select="provides" mode="decl-port-names"/>
  <xsl:apply-templates select="uses" mode="decl-port-names"/>

  <xsl:text>
  type(gov_cca_TypeMap_t)     :: myTypeMap
  type(SIDL_BaseInterface_t)  :: excpt
  type(gov_cca_Port_t)        :: myPort

  ! Access private data
  type(</xsl:text>
     <xsl:value-of select="concat($comp_typename, '_wrap) :: dp', $nl)"/>
     <xsl:text>  call </xsl:text>
     <xsl:value-of select="$comp_typename"/> <xsl:text>__get_data_m(self, dp)

  ! Set my reference to the services handle
  dp%d_private_data%frameworkServices = services
  ! Increment reference count for the services subroutine parameter
  call addRef(services)

  ! Create a TypeMap with my properties
  call createTypeMap(services, myTypeMap, excpt)</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:call-template name="checkException">
    <xsl:with-param name="msg" select="'in setServices call createTypeMap()'"/>
  </xsl:call-template>
  <xsl:text>  call cast(self, myPort)

  !
  ! Add the provided ports
  !</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>

  <!-- addProvidesPorts and registerUsesPorts -->

  <xsl:apply-templates select="provides" mode="add-provides-port"/>

  <xsl:text>
  !
  ! Register the used ports
  !</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>

  <xsl:apply-templates select="uses" mode="register-uses-port"/>

  <xsl:text>
  call deleteRef(myTypeMap)

  ! DO-NOT-DELETE splicer.end(</xsl:text>
  <xsl:value-of select="$comp_fullname"/> <xsl:text>.setServices)</xsl:text>
  <xsl:value-of select="$nl"/>

</xsl:template>


<!--
  - add-provides-port <provides>
  -->
<xsl:template match="provides" mode="add-provides-port">
  <xsl:text>  call addProvidesPort(services, myPort, pn_</xsl:text>
  <xsl:value-of select="@name"/> <xsl:text>, &amp;</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>                       pt_</xsl:text>
  <xsl:value-of select="concat(@name, ', myTypeMap, excpt)', $nl)"/>
  <xsl:call-template name="checkException">
    <xsl:with-param name="msg">
      <xsl:text>in setServices call addProvidesPort('</xsl:text>
      <xsl:value-of select="@name"/> <xsl:text>')</xsl:text>
    </xsl:with-param>
  </xsl:call-template>
</xsl:template>


<!--
  - register-uses-port <uses>
  -->
<xsl:template match="uses" mode="register-uses-port">
  <xsl:text>  call registerUsesPort(services, un_</xsl:text>
  <xsl:value-of select="@name"/> <xsl:text>, &amp;</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>                        ut_</xsl:text>
  <xsl:value-of select="concat(@name, ', myTypeMap, excpt)', $nl)"/>
  <xsl:call-template name="checkException">
    <xsl:with-param name="msg">
      <xsl:text>in setServices call registerUsesPort('</xsl:text>
      <xsl:value-of select="@name"/> <xsl:text>')</xsl:text>
    </xsl:with-param>
  </xsl:call-template>
  <xsl:value-of select="$nl"/>
</xsl:template>


<!--
  - decl-port-names <provides>
  -->
<xsl:template match="provides" mode="decl-port-names">
  <xsl:text>  character(len=*), parameter :: pn_</xsl:text>
  <xsl:value-of select="@name"/> <xsl:text> = '</xsl:text>
  <xsl:value-of select="@name"/> <xsl:text>'</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>  character(len=*), parameter :: pt_</xsl:text>
  <xsl:value-of select="@name"/> <xsl:text> = '</xsl:text>
  <xsl:value-of select="concat(@package, '.', @name)"/>
  <xsl:text>'</xsl:text> <xsl:value-of select="$nl"/>
</xsl:template>


<!--
  - decl-port-names <uses>
  -->
<xsl:template match="uses" mode="decl-port-names">
  <xsl:text>  character(len=*), parameter :: un_</xsl:text>
  <xsl:value-of select="@name"/> <xsl:text> = '</xsl:text>
  <xsl:value-of select="@name"/> <xsl:text>'</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>  character(len=*), parameter :: ut_</xsl:text>
  <xsl:value-of select="@name"/> <xsl:text> = '</xsl:text>
  <xsl:value-of select="concat(@package, '.', @name)"/>
  <xsl:text>'</xsl:text> <xsl:value-of select="$nl"/>
</xsl:template>


<!--
  - checkException: print code to check Babel exception
  -->
<xsl:template name="checkException">
  <xsl:param name="ws" select="'  '"/>
  <xsl:param name="excpt" select="'excpt'"/>
  <xsl:param name="msg" select="''"/>

  <xsl:value-of select="concat($ws, 'if (not_null(', $excpt, ')) then', $nl)"/>
  <xsl:value-of select="concat($ws, '  print *, &quot;')"/>
  <xsl:value-of select="concat($comp_fullname, ' Exception: ', $msg)"/>
  <xsl:value-of select="concat('&quot;', $nl)"/>
  <xsl:value-of select="concat($ws, '  call deleteRef(', $excpt, ')', $nl)"/>
  <xsl:value-of select="concat($ws, 'end if', $nl)"/>

</xsl:template>


<!--
  - openFile: print file header information
  -->
<xsl:template name="openFile"/>


<!--
  - closeFile: finish up
  -->
<xsl:template name="closeFile">

<xsl:text>

! DO-NOT-DELETE splicer.begin(_miscellaneous_code_end)
! Insert extra code here...
! DO-NOT-DELETE splicer.end(_miscellaneous_code_end)
</xsl:text>

</xsl:template>


</xsl:stylesheet>
