<!--
  - cca-f90.mod.xsl: creates a Babel _Mod.F90 file for a CCA component from a
  -                   Fortran module
  -
  - Output should be directed to the file, Package_Class_Mod.F90
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

<xsl:import href="common.xsl"/>
<xsl:import href="common-sidl.xsl"/>
<xsl:import href="common-f90.xsl"/>

<xsl:output method="text"/>

<!--
  - global variables:
  -   comp_name
  -   comp_package
  -   comp_fullname
  -   comp_typename
  -   interface_file
  -   impl_scope
  -
  - templates:
  -   
  -->

<xsl:variable name="comp_name" select="/componentImplementation/@name"/>
<xsl:variable name="comp_package"
              select="/componentImplementation/@package"/>
<xsl:variable name="comp_fullname">
  <xsl:value-of select="concat($comp_package, '.', $comp_name)"/>
</xsl:variable>
<xsl:variable name="comp_typename">
  <xsl:value-of select="concat($comp_package, '_', $comp_name)"/>
</xsl:variable>

<xsl:variable name="interface_file">
  <xsl:for-each
       select="/componentImplementation/language/property[@name='impl-xml']">
    <xsl:value-of select="@value"/>
  </xsl:for-each>
</xsl:variable>

<xsl:variable name="impl_scope">
  <xsl:for-each
       select="/componentImplementation/language/property[@name='impl-scope']">
    <xsl:value-of select="@value"/>
  </xsl:for-each>
</xsl:variable>


<!--
 -  /componentImplementation node
 -->
<xsl:template match="/componentImplementation">
  <xsl:call-template name="openFile"/>

  <xsl:call-template name="splicer-use"/>
  <xsl:call-template name="splicer-private-data"/>

  <xsl:call-template name="closeFile"/>
</xsl:template>


<!--
  - splicer-use </componentImplementation>
  -->
<xsl:template name="splicer-use">
  <xsl:text>
! DO-NOT-DELETE splicer.begin(</xsl:text>
  <xsl:value-of select="$comp_fullname"/> <xsl:text>.use)
! Insert use statements here...

! CCA framework services module
use gov_cca_Services

! DO-NOT-DELETE splicer.end(</xsl:text>
  <xsl:value-of select="$comp_fullname"/> <xsl:text>.use)</xsl:text>
  <xsl:value-of select="$nl"/>

</xsl:template>


<!--
  - splicer-private-data </componentImplementation>
  -->
<xsl:template name="splicer-private-data">
  <xsl:text>
  ! DO-NOT-DELETE splicer.begin(</xsl:text>
  <xsl:value-of select="$comp_fullname"/> <xsl:text>.private_data)

  ! Handle to framework Services object
  type(gov_cca_Services_t) :: frameworkServices

  ! DO-NOT-DELETE splicer.end(</xsl:text>
  <xsl:value-of select="$comp_fullname"/> <xsl:text>.private_data)</xsl:text>
  <xsl:value-of select="$nl"/>

</xsl:template>


<!--
  - openFile: print file header information
  -->
<xsl:template name="openFile"/>


<!--
  - closeFile: finish up
  -->
<xsl:template name="closeFile"/>


</xsl:stylesheet>
