<!-- LANL:license
 ...........................................................................
 - This SOFTWARE has been authored by an employee or employees of the
 - University of California, operator of the Los Alamos National Laboratory
 - under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 - The U.S. Government has rights to use, reproduce, and distribute this
 - SOFTWARE.  The public may copy, distribute, prepare derivative works and
 - publicly display this SOFTWARE without charge, provided that this Notice
 - and any statement of authorship are reproduced on all copies.  Neither
 - the Government nor the University makes any warranty, express or implied,
 - or assumes any liability or responsibility for the use of this SOFTWARE.
 - If SOFTWARE is modified to produce derivative works, such modified
 - SOFTWARE should be clearly marked, so as not to confuse it with the
 - version available from LANL.
 ...........................................................................
 - LANL:license
 ...........................................................................
 -->

<!--
  - chasm-mpi.f90.xsl: creates F90 functions that call MPI equivalent
  -
  - Output should be directed to the file, Method_f90.f90
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

<xsl:import href="common.xsl"/>
<xsl:import href="common-C.xsl"/>
<xsl:import href="common-f90.xsl"/>

<xsl:output method="text"/>

<!-- global variables -->

<xsl:variable name="filename">
  <xsl:call-template name="lower-case">
    <xsl:with-param name="symbol" select="/library/scope/@name"/>
  </xsl:call-template>
  <xsl:text>_C.c</xsl:text>
</xsl:variable>

<xsl:variable name="header_file">
  <xsl:call-template name="lower-case">
    <xsl:with-param name="symbol" select="/library/scope/@name"/>
  </xsl:call-template>
  <xsl:text>_C.h</xsl:text>
</xsl:variable>


<!--
  - root level
  -->
<xsl:template match="/">
  <xsl:call-template name="openFile">
    <xsl:with-param name="filename" select="$filename"/>
  </xsl:call-template>

  <!-- define interfaces -->

  <xsl:for-each select="library/scope">
    <xsl:call-template name="defineInterfaces"/>
  </xsl:for-each>

  <xsl:call-template name="closeFile">
    <xsl:with-param name="filename" select="$filename"/>
  </xsl:call-template>
</xsl:template>


<!--
  - defineFunctions: define BIND(C) interfaces to Fortran procedures <scope>
  -->
<xsl:template name="defineInterfaces">
  <xsl:param name="module" select="@name"/>
  <xsl:value-of select="$nl"/>
  <xsl:for-each select="method">

    <xsl:text>INTERFACE</xsl:text>
    <xsl:value-of select="$nl"/>

    <xsl:call-template name="defineInterface"/>

    <xsl:text>END INTERFACE</xsl:text>
    <xsl:value-of select="$nl"/>
    <xsl:value-of select="$nl"/>

  </xsl:for-each>

</xsl:template>


<!--
  - defineInterface <method>
  -->
<xsl:template name="defineInterface">
  <xsl:variable name="ws" select="'    '"/>

  <xsl:call-template name="def-proc-begin">
    <xsl:with-param name="ws" select="$ws"/>
  </xsl:call-template>

  <xsl:text>    USE, INTRINSIC :: ISO_C_BINDING</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:call-template name="decl-construct-list">
    <xsl:with-param name="ws" select="$ws"/>
  </xsl:call-template>

  <xsl:call-template name="decl-result">
    <xsl:with-param name="ws" select="$ws"/>
  </xsl:call-template>

  <xsl:call-template name="def-proc-end">
    <xsl:with-param name="ws" select="$ws"/>
  </xsl:call-template>

</xsl:template>


<!--
  - def-proc-begin <method>
  -->
<xsl:template name="def-proc-begin">
  <xsl:variable name="ws" select="'    '"/>

  <xsl:choose>
    <xsl:when test="return/type/@kind = 'void'">
      <xsl:value-of select="concat('  SUBROUTINE ', @name, '(')"/>
      <xsl:call-template name="arg-list"/> <xsl:text>)</xsl:text>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="concat('  FUNCTION ', @name, '(')"/>
      <xsl:call-template name="arg-list"/>
      <xsl:text>) RESULT(rtn)</xsl:text>
    </xsl:otherwise>
  </xsl:choose>
  <xsl:text> BIND(C,NAME="</xsl:text>
  <xsl:value-of select="@name"/>
  <xsl:text>")</xsl:text>
  <xsl:value-of select="$nl"/>

</xsl:template>


<!--
  - def-proc-end <method>
  -->
<xsl:template name="def-proc-end">
  <xsl:variable name="ws" select="'    '"/>

  <xsl:choose>
    <xsl:when test="return/type/@kind = 'void'">
      <xsl:value-of select="concat('  END SUBROUTINE ', @name)"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="concat('  END FUNCTION ', @name)"/>
    </xsl:otherwise>
  </xsl:choose>
  <xsl:value-of select="$nl"/>

</xsl:template>


<!--
  - arg-list <method>
  -->
<xsl:template name="arg-list">
  <xsl:for-each select="arg">
    <xsl:value-of select="@name"/>
    <xsl:if test="position() != last()">
      <xsl:text>, </xsl:text>
    </xsl:if>
    <xsl:if test="position() = 5">
      <xsl:text>&amp;</xsl:text>
      <xsl:value-of select="concat($nl, '        ')"/>
    </xsl:if>
  </xsl:for-each>
</xsl:template>


<!--
  - decl-construct-list <method>
  -->
<xsl:template name="decl-construct-list">
  <xsl:param name="ws" select="'  '"/>
  <xsl:for-each select="arg">
    <xsl:call-template name="type-decl-stmt">
      <xsl:with-param name="ws" select="$ws"/>
    </xsl:call-template>
    <xsl:value-of select="concat(' :: ', @name)"/>
    <xsl:for-each select="type/array">
      <xsl:call-template name="decl-type-spec-array-dim"/>
    </xsl:for-each>
    <xsl:value-of select="$nl"/>
  </xsl:for-each>
</xsl:template>


<!--
  - decl-result <method>
  -->
<xsl:template name="decl-result">
  <xsl:param name="ws" select="'  '"/>
  <xsl:for-each select="return">
    <xsl:if test="type/@kind != 'void'">
      <xsl:call-template name="type-decl-stmt">
        <xsl:with-param name="ws" select="$ws"/>
      </xsl:call-template>
      <xsl:text> :: rtn</xsl:text>
      <xsl:value-of select="$nl"/>
    </xsl:if>
  </xsl:for-each>
</xsl:template>


<!--
  - type-decl-stmt <arg>
  -->
<xsl:template name="type-decl-stmt">
  <xsl:param name="ws" select="'  '"/>

  <xsl:value-of select="$ws"/>
  <xsl:for-each select="type[1]">
    <xsl:call-template name="decl-type-spec"/>
    <xsl:call-template name="decl-type-spec-attr-value"/>
    <xsl:call-template name="decl-type-spec-attr-intent"/>
  </xsl:for-each>

</xsl:template>


<!--
  - decl-type-spec <arg>
  -->
<xsl:template name="decl-type-spec">

  <xsl:choose>

    <!-- C++ types -->

    <xsl:when test="@kind = 'void'">
      <xsl:text>TYPE(C_PTR)</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'int'">
      <xsl:choose>
        <xsl:when test="@ikind = 'char'">
          <xsl:text>CHARACTER(C_CHAR)</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'uchar'">
          <xsl:text>CHARACTER(C_CHAR)</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'short'">
          <xsl:text>INTEGER(C_SHORT)</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'ushort'">
          <xsl:text>INTEGER(C_SHORT)</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'int'">
          <xsl:text>INTEGER(C_INT)</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'uint'">
          <xsl:text>INTEGER(C_INT)</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'long'">
          <xsl:text>INTEGER(C_LONG)</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'ulong'">
          <xsl:text>INTEGER(C_LONG)</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'longlong'">
          <xsl:text>INTEGER(C_LONG_LONG)</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'ulonglong'">
          <xsl:text>INTEGER(C_LONG_LONG)</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>UNSUPPORTED</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:when test="@kind = 'float'">
      <xsl:choose>
        <xsl:when test="@fkind = 'float'">
          <xsl:text>REAL(C_FLOAT)</xsl:text>
        </xsl:when>
        <xsl:when test="@fkind = 'dbl'">
          <xsl:text>REAL(C_FLOAT)</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>UNSUPPORTED</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:when test="@kind = 'ptr'">
      <xsl:for-each select="indirect/type">
        <xsl:call-template name="type-spec-ptr">
          <xsl:with-param name="depth" select="../@depth"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:when>
    <xsl:when test="@kind = 'ref'">
      <xsl:for-each select="indirect/type">
        <xsl:call-template name="type-spec">
          <xsl:with-param name="depth" select="../@depth"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:when>

    <!-- user-defined types -->

    <xsl:when test="@kind = 'usertype'">
      <xsl:call-template name="type-spec-user">
        <xsl:with-param name="out_default">
          <xsl:value-of select="concat('TYPE(', @usertype, ')')"/>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:when>


    <!-- Fortran types -->

    <xsl:when test="@kind = 'fint'">
      <xsl:text>integer</xsl:text>
      <xsl:choose>
        <xsl:when test="@ikind = 'char'">
          <xsl:text>(C_SIGNED_CHAR)</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'short'">
          <xsl:text>(C_SHORT)</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'int'">
          <xsl:text>(C_INT)</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'long'">
          <xsl:text>(C_LONG)</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'longlong'">
          <xsl:text>(C_LONG_LONG)</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>(UNSUPPORTED)</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:when test="@kind = 'ffloat'">
      <xsl:text>real</xsl:text>
      <xsl:choose>
        <xsl:when test="@fkind = 'float'">
          <xsl:text>(C_FLOAT)</xsl:text>
        </xsl:when>
        <xsl:when test="@fkind = 'dbl'">
          <xsl:text>(C_DOUBLE)</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>UNSUPPORTED</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:when test="@kind = 'fcmplx'">
      <xsl:text>complex</xsl:text>
      <xsl:choose>
        <xsl:when test="@fkind = 'float'">
          <xsl:text>(C_FLOAT_COMPLEX)</xsl:text>
        </xsl:when>
        <xsl:when test="@fkind = 'dbl'">
          <xsl:text>(C_DOUBLE_COMPLEX)</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>UNSUPPORTED</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:when test="@kind = 'flogic'">
      <xsl:text>logical(C_BOOL)</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'fchar'">
      <xsl:text>character(C_CHAR)</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'farray'">
      <xsl:for-each select="array/type">
        <xsl:call-template name="decl-type-spec"/>
      </xsl:for-each>
    </xsl:when>
    <xsl:when test="@kind = 'fptr'"/>
    <xsl:otherwise>
      <xsl:text>UNSUPPORTED</xsl:text>
    </xsl:otherwise>
  </xsl:choose>

</xsl:template>


<!--
  - type-spec-ptr <type> - is a pointer
  -->
<xsl:template name="type-spec-ptr">
  <xsl:param name="depth" select="0"/>

  <xsl:choose>
    <xsl:when test="@kind = 'usertype'">
      <xsl:variable name="user_defined_type">
        <xsl:call-template name="type-spec-user">
          <xsl:with-param name="out_default">
            <xsl:value-of select="'UNSUPPORTED'"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:variable>
      <xsl:choose>
        <xsl:when test="$user_defined_type = 'UNSUPPORTED'">
          <xsl:text>TYPE(C_PTR), VALUE</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$user_defined_type"/>
          <xsl:text>, DIMENSION(*)</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="decl-type-spec"/>
    </xsl:otherwise>
  </xsl:choose>

</xsl:template>


<!--
  - decl-type-spec-attr-value <arg>
  -->
<xsl:template name="decl-type-spec-attr-value">
  <xsl:choose>

    <!-- C++ types -->

    <xsl:when test="@kind = 'void'">
      <xsl:text>, VALUE</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'int'">
      <xsl:text>, VALUE</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'float'">
      <xsl:text>, VALUE</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'ptr'">
      <xsl:for-each select="indirect/type">
        <xsl:call-template name="decl-type-spec-attr-value"/>
      </xsl:for-each>
    </xsl:when>
    <xsl:when test="@kind = 'ref'"/>

    <!-- handled elsewhere -->
    <xsl:when test="@kind = 'usertype'"/>

    <!-- Fortran types -->

    <xsl:when test="@kind = 'fint'">
      <xsl:text>, VALUE</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'ffloat'">
      <xsl:text>, VALUE</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'fcmplx'"/>
    <xsl:when test="@kind = 'flogic'">
      <xsl:text>, VALUE</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'fchar'">
      <xsl:text>, VALUE</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'farray'"/>
    <xsl:when test="@kind = 'fptr'"/>
    <xsl:otherwise/>
  </xsl:choose>
</xsl:template>


<!--
  - decl-type-spec-attr-intent <arg>
  -->
<xsl:template name="decl-type-spec-attr-intent">
  <xsl:choose>
    <xsl:when test="@const = 'yes'">
      <xsl:text>, INTENT(IN)</xsl:text>
    </xsl:when>

    <xsl:when test="@kind = 'usertype'">
      <xsl:for-each select="indirect/type">
        <xsl:if test="@const = 'yes'">
          <xsl:text>, VALUE</xsl:text>
        </xsl:if>
      </xsl:for-each>
    </xsl:when>

    <xsl:otherwise/>
  </xsl:choose>
</xsl:template>


<!--
  - decl-type-spec-array-dim <array>
  -->
<xsl:template name="decl-type-spec-array-dim">
  <xsl:text>(</xsl:text>
  <xsl:for-each select="dimension">
    <xsl:choose>
      <xsl:when test="@alias">
        <xsl:value-of select="@alias"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="@extent"/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:if test="position() != last()">
      <xsl:text>,</xsl:text>
    </xsl:if>
  </xsl:for-each>
  <xsl:text>)</xsl:text>
</xsl:template>


<!--
  - openFile: print file header information
  -->
<xsl:template name="openFile">
  <xsl:param name="filename" select="''"/>
</xsl:template>


<!--
  - closeFile: finish up
  -->
<xsl:template name="closeFile">
  <xsl:param name="filename" select="''"/>
</xsl:template>


</xsl:stylesheet>
