<!-- LANL:license
 ...........................................................................
 - This SOFTWARE has been authored by an employee or employees of the
 - University of California, operator of the Los Alamos National Laboratory
 - under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 - The U.S. Government has rights to use, reproduce, and distribute this
 - SOFTWARE.  The public may copy, distribute, prepare derivative works and
 - publicly display this SOFTWARE without charge, provided that this Notice
 - and any statement of authorship are reproduced on all copies.  Neither
 - the Government nor the University makes any warranty, express or implied,
 - or assumes any liability or responsibility for the use of this SOFTWARE.
 - If SOFTWARE is modified to produce derivative works, such modified
 - SOFTWARE should be clearly marked, so as not to confuse it with the
 - version available from LANL.
 ...........................................................................
 - LANL:license
 ...........................................................................
 -->

<!--
  - chasm-f90-py.c.xsl: creates a extension module file for Fortran
  -   module procedures.
  -
  - Output should be directed to the file, MODULE_py.c
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

<xsl:import href="common.xsl"/>
<xsl:import href="common-C.xsl"/>
<xsl:import href="common-f90.xsl"/>
<xsl:import href="common-py.xsl"/>
<xsl:import href="common-py-f90.xsl"/>
<xsl:import href="type-conv-py-f90.xsl"/>

<xsl:output method="text"/>

<!-- global variables -->

<xsl:variable name="filename">
  <xsl:call-template name="lower-case">
    <xsl:with-param name="symbol" select="/library/scope/@name"/>
  </xsl:call-template>
  <xsl:text>_py.c</xsl:text>
</xsl:variable>


<!--
  - root level
  -->
<xsl:template match="/">
  <xsl:call-template name="openFile">
    <xsl:with-param name="filename" select="$filename"/>
  </xsl:call-template>

  <!-- output all include files -->

  <xsl:call-template name="include-files">
    <xsl:with-param name="filename" select="$filename"/>
  </xsl:call-template>

  <!-- define struct for Python object -->

  <xsl:for-each select="library/scope">
    <xsl:call-template name="definePyStruct"/>
  </xsl:for-each>

  <!-- define macros for Fortran procedure symbols -->

  <xsl:for-each select="library/scope">
    <xsl:call-template name="defineMacros">
      <xsl:with-param name="symbol">
        <xsl:text>CH_INIT_</xsl:text> <xsl:value-of select="@name"/>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:for-each>

  <xsl:call-template name="openExternC"/>

  <!-- declare Fortran initialize function -->

  <xsl:for-each select="library/scope">
    <xsl:call-template name="declareInitFunction"/>
  </xsl:for-each>

  <!-- declare pointers to Fortran procedures -->

  <xsl:for-each select="library/scope">
    <xsl:call-template name="declareFunctionPointers"/>
  </xsl:for-each>
  <xsl:value-of select="$nl"/>

  <!-- define functions to set pointers to Fortran procedures -->

  <xsl:for-each select="library/scope">
    <xsl:call-template name="defineSetFunctionPointers"/>
  </xsl:for-each>

  <!-- define Python-C bridging functions -->

  <xsl:for-each select="library/scope">
    <xsl:call-template name="defineFunctions"/>
  </xsl:for-each>

  <!-- define Python Members -->

  <xsl:for-each select="library/scope">
    <xsl:call-template name="definePyMembers"/>
  </xsl:for-each>

  <!-- define PyMethodDef -->

  <xsl:for-each select="library/scope">
    <xsl:call-template name="definePyMethodDef"/>
  </xsl:for-each>

  <!-- define Python module constructor -->

  <xsl:for-each select="library/scope">
    <xsl:call-template name="definePyNew"/>
  </xsl:for-each>

  <!-- define Module init -->

  <xsl:for-each select="library/scope">
    <xsl:call-template name="definePyInit"/>
  </xsl:for-each>

  <!-- define Python module destructor -->

  <xsl:for-each select="library/scope">
    <xsl:call-template name="definePyDealloc"/>
  </xsl:for-each>

  <!-- define PyTypeObject -->

  <xsl:for-each select="library/scope">
    <xsl:call-template name="definePyTypeObject"/>
  </xsl:for-each>

  <!-- define PyModuleInit -->

  <xsl:for-each select="library/scope">
    <xsl:call-template name="definePyModuleInit"/>
  </xsl:for-each>

  <xsl:call-template name="closeExternC"/>

  <xsl:call-template name="closeFile">
    <xsl:with-param name="filename" select="$filename"/>
  </xsl:call-template>

</xsl:template>

</xsl:stylesheet>
