<!-- LANL:license
 ...........................................................................
 - This SOFTWARE has been authored by an employee or employees of the
 - University of California, operator of the Los Alamos National Laboratory
 - under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 - The U.S. Government has rights to use, reproduce, and distribute this
 - SOFTWARE.  The public may copy, distribute, prepare derivative works and
 - publicly display this SOFTWARE without charge, provided that this Notice
 - and any statement of authorship are reproduced on all copies.  Neither
 - the Government nor the University makes any warranty, express or implied,
 - or assumes any liability or responsibility for the use of this SOFTWARE.
 - If SOFTWARE is modified to produce derivative works, such modified
 - SOFTWARE should be clearly marked, so as not to confuse it with the
 - version available from LANL.
 ...........................................................................
 - LANL:license
 ...........................................................................
 -->

<!--
  - chasm-f90.h.xsl: creates a header file for C functions that call
  -   Fortran module procedures.
  -
  - Output should be directed to the file, MODULE_C.h
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

<xsl:import href="common.xsl"/>
<xsl:import href="common-C.xsl"/>
<xsl:import href="common-f90.xsl"/>

<xsl:output method="text"/>

<!-- global variables -->

<xsl:variable name="filename">
  <xsl:call-template name="lower-case">
    <xsl:with-param name="symbol" select="/library/scope/@name"/>
  </xsl:call-template>
  <xsl:text>_C.h</xsl:text>
</xsl:variable>


<!--
  - root level
  -->
<xsl:template match="/">
  <xsl:call-template name="openFile">
    <xsl:with-param name="filename" select="$filename"/>
  </xsl:call-template>

  <!-- output all include files -->

  <xsl:call-template name="include-files">
    <xsl:with-param name="filename" select="$filename"/>
  </xsl:call-template>

  <!-- define macros for Fortran procedure symbols -->

  <xsl:for-each select="library/scope">
    <xsl:call-template name="defineInitMacro"/>
  </xsl:for-each>

  <xsl:call-template name="openExternC"/>

  <!-- declare functions -->

  <xsl:for-each select="library/scope/method">
    <xsl:call-template name="function-decl">
      <xsl:with-param name="id">
        <xsl:value-of select="../@name"/> <xsl:text>_</xsl:text>
        <xsl:value-of select="@name"/>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:for-each>
  <xsl:value-of select="$nl"/>

  <xsl:call-template name="closeExternC"/>

  <xsl:call-template name="closeFile">
    <xsl:with-param name="filename" select="$filename"/>
  </xsl:call-template>
</xsl:template>


<!--
  - include-files: include header files
  -->
<xsl:template name="include-files">
  <xsl:call-template name="include-file">
    <xsl:with-param name="filename" select="'F90Compiler.h'"/>
  </xsl:call-template>
  <xsl:call-template name="include-file">
    <xsl:with-param name="filename" select="'F90Complex.h'"/>
  </xsl:call-template>
  <xsl:value-of select="$nl"/>
</xsl:template>


</xsl:stylesheet>
