<!-- LANL:license
 ...........................................................................
 - This SOFTWARE has been authored by an employee or employees of the
 - University of California, operator of the Los Alamos National Laboratory
 - under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 - The U.S. Government has rights to use, reproduce, and distribute this
 - SOFTWARE.  The public may copy, distribute, prepare derivative works and
 - publicly display this SOFTWARE without charge, provided that this Notice
 - and any statement of authorship are reproduced on all copies.  Neither
 - the Government nor the University makes any warranty, express or implied,
 - or assumes any liability or responsibility for the use of this SOFTWARE.
 - If SOFTWARE is modified to produce derivative works, such modified
 - SOFTWARE should be clearly marked, so as not to confuse it with the
 - version available from LANL.
 ...........................................................................
 - LANL:license
 ...........................................................................
 -->

<!--
  - common templates for creating C++ files
  -
  - templates:
  -
  - decl-construct-list ([ws='  ']) <method>
  - type-decl-stmt <arg>
  -
  - assign-stmt-list ([ws='  ']) <method>
  - assign-stmt <arg>
  -
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

<xsl:import href="type-conv-cpp.xsl"/>


<!--
  - type-conv-name-ext <arg/type>
  -->
<xsl:template name="type-conv-name-ext">
  <xsl:param name="depth" select="0"/>
  <xsl:choose>
    <!-- Fortran types -->
    <xsl:when test="@kind = 'farray'">
      <xsl:if test="array/dimension/@extent = '*'">
        <xsl:text>_dv</xsl:text>
      </xsl:if>
    </xsl:when>
  </xsl:choose>
</xsl:template>

<!--
  - decl-construct-list <method>
  -->
<xsl:template name="decl-construct-list">
  <xsl:param name="ws" select="'  '"/>
  <xsl:for-each select="arg">
    <xsl:call-template name="type-decl-stmt">
      <xsl:with-param name="ws" select="$ws"/>
    </xsl:call-template>
  </xsl:for-each>
</xsl:template>


<!--
  - type-decl-stmt <arg>
  -->
<xsl:template name="type-decl-stmt">
  <xsl:param name="ws" select="'  '"/>

  <xsl:value-of select="$ws"/>
  <xsl:for-each select="type[1]">
    <xsl:call-template name="decl-type-spec"/>
  </xsl:for-each>

  <xsl:value-of select="concat(', intent(', @intent, ')')"/>
  <xsl:value-of select="concat(' :: ', @name, '_l', $nl)"/>

</xsl:template>


<!--
  - assign-stmt-list <method>
  -->
<xsl:template name="assign-stmt-list">
  <xsl:param name="ws" select="'  '"/>
  <xsl:for-each select="arg">
    <xsl:call-template name="assign-stmt">
      <xsl:with-param name="ws" select="$ws"/>
    </xsl:call-template>
  </xsl:for-each>
</xsl:template>


<!--
  - assign-stmt <arg>
  -->
<xsl:template name="assign-stmt">
  <xsl:param name="ws" select="'  '"/>
  <xsl:param name="lowercase_name">
    <xsl:call-template name="lower-case">
      <xsl:with-param name="symbol" select="@name"/>
    </xsl:call-template>
  </xsl:param>

  <xsl:value-of select="$ws"/>
  <xsl:value-of select="concat(@name, '_l = ', $lowercase_name)"/>
  <xsl:value-of select="$nl"/>

</xsl:template>


</xsl:stylesheet>
