<!--
  - common templates for creating Python wrappers
  -
  - templates:
  -
  - include-files: include header files
  -
  - param-id-list (method) <method>
  - decl-list <method>
  - decl-list-decl-arg <arg/type>
  - statement-list <method>
  -
  - definePyStruct <library/scope>
  - decl-list-struct ([name]) <scope>
  - definePyMembers <scope>
  - definePyMethodDef <scope>
  - definePyNew <scope>
  - definePyInit <scope>
  - definePyDealloc <scope>
  - definePyTypePobject <scope>
  - definePyModuleInit
  -
  -->

<xsl:stylesheet
     xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
     version="1.0">

<xsl:import href="type-conv-py.xsl"/>
<xsl:output method="text"/>


<!--
  - include-files: include header files
  -->
<xsl:template name="include-files">
  <xsl:call-template name="include-file">
    <xsl:with-param name="filename" select="'Python.h'"/>
  </xsl:call-template>
  <xsl:call-template name="include-file">
    <xsl:with-param name="filename" select="'structmember.h'"/>
  </xsl:call-template>
  <xsl:call-template name="include-file">
    <xsl:with-param name="filename" select="'F90Compiler.h'"/>
  </xsl:call-template>
  <xsl:call-template name="include-file">
    <xsl:with-param name="filename" select="'F90Complex.h'"/>
  </xsl:call-template>
  <xsl:value-of select="$nl"/>
</xsl:template>


<!--
  - param-id-list (module) <method>
  -->
<xsl:template name="param-id-list">
  <xsl:param name="module"/> 
  <xsl:value-of select="$module"/>
  <xsl:text>* self, PyObject* args</xsl:text>
</xsl:template>


<!--
  - decl-list <method>
  -->
<xsl:template name="decl-list">
  <xsl:text>  PyObject* result_ = Py_None;</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>  int ok_ = 1;</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>
  <xsl:for-each select="return[1]/type">
    <xsl:call-template name="decl-list-decl-rtn"/>
  </xsl:for-each>
  <xsl:for-each select="arg/type">
    <xsl:call-template name="decl-list-decl-arg"/>
  </xsl:for-each>
  <xsl:value-of select="$nl"/>
</xsl:template>


<!--
  - decl-list-decl-arg <arg/type>
  -->
<xsl:template name="decl-list-decl-arg">
  <xsl:variable name="arg_id">
    <xsl:call-template name="arg-id"/>
  </xsl:variable>
  <xsl:variable name="type_spec_t"/>
  <xsl:variable name="ws" select="'  '"/>
  <xsl:if test='string-length($arg_id) &gt; 0'>
    <xsl:value-of select="$ws"/> <xsl:call-template name="type-spec-py"/>
    <xsl:text> </xsl:text> <xsl:value-of select="$arg_id"/>
    <xsl:text>;</xsl:text> <xsl:value-of select="$nl"/>
  </xsl:if>
</xsl:template>


<!--
  - statement-list <method>
  -->
<xsl:template name="statement-list">
  <xsl:param name="function_id">
    <xsl:call-template name="function-id"/>
  </xsl:param>
  <xsl:if test="arg">
    <xsl:text>  ok_ = PyArg_ParseTuple(args, "</xsl:text>
    <xsl:for-each select="arg/type">
      <xsl:call-template name="type-conv-str"/>
    </xsl:for-each>
    <xsl:text>", </xsl:text>
    <xsl:call-template name="arg-list-ParseTuple"/>
    <xsl:text>);</xsl:text>
    <xsl:value-of select="$nl"/>
  </xsl:if>
  <xsl:text>  if (ok_) {</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:for-each select="arg/type">
    <xsl:call-template name="decl-list-decl-arg-f90">
      <xsl:with-param name="ws" select="'    '"/>
    </xsl:call-template>
  </xsl:for-each> 
  <xsl:call-template name="statement-list-pre-call">
    <xsl:with-param name="ws" select="'    '"/>
  </xsl:call-template>
  <xsl:text>  </xsl:text>
  <xsl:call-template name="call-statement">
    <xsl:with-param name="id">
      <xsl:text>PTR_</xsl:text> <xsl:value-of select="$function_id"/>
    </xsl:with-param>
  </xsl:call-template>
  <xsl:call-template name="call-statement-rtn-conv">
  </xsl:call-template>
  <xsl:text>  } else {</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>    PyErr_SetString(PyExc_Exception,
          "ERROR converting Python args for </xsl:text>
  <xsl:value-of select="$function_id"/> <xsl:text>");</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>    return 0;</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>  }</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>  Py_XINCREF(result_);</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>  return result_;</xsl:text> <xsl:value-of select="$nl"/>
</xsl:template>


<!--
  - arg-list-ParseTuple <method>
  -->
<xsl:template name="arg-list-ParseTuple">
  <xsl:for-each select="arg/type">
    <xsl:call-template name="arg-id-conv-ParseTuple"/>
    <xsl:call-template name="address-operator-ParseTuple"/>
    <xsl:call-template name="arg-id"/>
    <xsl:if test="position() != last()">
      <xsl:text>, </xsl:text>
    </xsl:if>
  </xsl:for-each>
</xsl:template>


<!--
  - call-statement-rtn-conv <method>
  -->
<xsl:template name="call-statement-rtn-conv">
  <xsl:param name="ws" select="'    '"/>
  <xsl:value-of select="$ws"/>
  <xsl:variable name="conv_str">
    <xsl:for-each select="arg/type">
      <xsl:call-template name="type-conv-str-rtn"/>
    </xsl:for-each>
  </xsl:variable>
  <xsl:if test="$conv_str != ''">
    <xsl:text>result_ = Py_BuildValue("(</xsl:text>
    <xsl:value-of select="$conv_str"/>
    <xsl:text>)"</xsl:text>
    <xsl:for-each select="arg/type">
      <xsl:variable name="conv_str_single">
        <xsl:call-template name="type-conv-str-rtn"/>
      </xsl:variable>
      <xsl:if test="$conv_str_single != ''">
        <xsl:text>, </xsl:text>
        <xsl:value-of select="../@name"/>
      </xsl:if>
    </xsl:for-each>
    <xsl:text>);</xsl:text> <xsl:value-of select="$nl"/>
  </xsl:if>
</xsl:template>


<!--
  - definePyStruct <scope>
  -->
<xsl:template name="definePyStruct">
  <xsl:param name="name" select="@name"/>
  <xsl:text>typedef struct {</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:call-template name="decl-list-struct"/>
  <xsl:text>} </xsl:text> <xsl:value-of select="$name"/>
  <xsl:text>;</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>
</xsl:template>	


<!--
  - decl-list-struct
  -->
<xsl:template name="decl-list-struct">
  <xsl:text>  PyObject_HEAD</xsl:text> <xsl:value-of select="$nl"/>
</xsl:template>


<!--
  - definePyMembers <scope>
  -->
<xsl:template name="definePyMembers">
  <xsl:param name="name" select="@name"/>
  <xsl:text>static PyMemberDef </xsl:text> <xsl:value-of select="$name"/>
  <xsl:text>_members[] = {</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>  {NULL}  /* Sentinel */</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>};</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>
</xsl:template>	


<!--
  - definePyMethodDef <scope>
  -->
<xsl:template name="definePyMethodDef">
  <xsl:param name="name" select="@name"/>
  <xsl:text>static PyMethodDef </xsl:text> <xsl:value-of select="$name"/>
  <xsl:text>_methods[] = {</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:for-each select="method">
    <xsl:variable name="lc_method_name">
      <xsl:call-template name="lower-case">
        <xsl:with-param name="symbol" select="@name"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:text>  {"</xsl:text> <xsl:value-of select="$lc_method_name"/>
    <xsl:text>", (PyCFunction)</xsl:text> <xsl:value-of select="$name"/>
    <xsl:text>_</xsl:text> <xsl:value-of select="@name"/>
    <xsl:text>, METH_VARARGS,</xsl:text> <xsl:value-of select="$nl"/>
    <xsl:text>   "A method of the </xsl:text> <xsl:value-of select="$name"/>
    <xsl:text> module"</xsl:text> <xsl:value-of select="$nl"/>
    <xsl:text>  },</xsl:text> <xsl:value-of select="$nl"/>
  </xsl:for-each>
  <xsl:text>  {NULL}  /* Sentinel */</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>};</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>
</xsl:template>	


<!--
  - definePyNew <scope>
  -->
<xsl:template name="definePyNew">
  <xsl:param name="name" select="@name"/>
  <xsl:text>static PyObject*</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:value-of select="$name"/>
  <xsl:text>_new(PyTypeObject* type, PyObject* args, PyObject* kwds)</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>{</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>  </xsl:text><xsl:value-of select="$name"/>
  <xsl:text>* self;</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>  self = (</xsl:text> <xsl:value-of select="$name"/>
  <xsl:text>*) type->tp_alloc(type, 0);</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>  return (PyObject*) self;</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>}</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>
</xsl:template>


<!--
  - definePyInit <scope>
  -->
<xsl:template name="definePyInit">
  <xsl:param name="name" select="@name"/>
  <xsl:param name="language" select="'F90'"/>
  <xsl:text>static int</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:value-of select="$name"/> <xsl:text>_init(</xsl:text>
  <xsl:value-of select="$name"/>
  <xsl:text>* self, PyObject* args, PyObject* kwds)</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>{</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:if test="$language = 'F90'">
    <xsl:text>  CH_INIT_</xsl:text> <xsl:value-of select="$name"/>
    <xsl:text>();</xsl:text> <xsl:value-of select="$nl"/>
  </xsl:if>
  <xsl:text>  return 0;</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>}</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>
</xsl:template>	


<!--
  - definePyDealloc <scope>
  -->
<xsl:template name="definePyDealloc">
  <xsl:param name="name" select="@name"/>
  <xsl:text>static void</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:value-of select="$name"/> <xsl:text>_dealloc(</xsl:text>
  <xsl:value-of select="$name"/> <xsl:text>* self)</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>{</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>  self->ob_type->tp_free((PyObject*) self);</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>}</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>
</xsl:template>	


<!--
  - definePyTypePobject <scope>
  -->
<xsl:template name="definePyTypeObject">
  <xsl:param name="name" select="@name"/>
  <xsl:param name="language" select="'F90'"/>
  <xsl:param name="type" select="@name"/>
  <xsl:variable name="type_name" select="concat($language, '.', $type)"/>
  <xsl:text>static PyTypeObject </xsl:text> <xsl:value-of select="$name"/>
  <xsl:text>Type = {
  PyObject_HEAD_INIT(NULL)
  0,                         /*ob_size*/
  &quot;</xsl:text>  <xsl:value-of select="$type_name"/>
          <xsl:text>&quot;,  /*tp_name*/
  sizeof(</xsl:text> <xsl:value-of select="$name"/>
               <xsl:text>),  /*tp_basicsize*/
  0,                         /*tp_itemsize*/
  (destructor)</xsl:text> <xsl:value-of select="$name"/>
         <xsl:text>_dealloc, /*tp_dealloc*/
  0,                         /*tp_print*/
  0,                         /*tp_getattr*/
  0,                         /*tp_setattr*/
  0,                         /*tp_compare*/
  0,                         /*tp_repr*/
  0,                         /*tp_as_number*/
  0,                         /*tp_as_sequence*/
  0,                         /*tp_as_mapping*/
  0,                         /*tp_hash */
  0,                         /*tp_call*/
  0,                         /*tp_str*/
  0,                         /*tp_getattro*/
  0,                         /*tp_setattro*/
  0,                         /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, /*tp_flags*/
  &quot;</xsl:text> <xsl:value-of select="$name"/>
   <xsl:text> objects&quot;, /* tp_doc */
  0,                         /* tp_traverse */
  0,                         /* tp_clear */
  0,                         /* tp_richcompare */
  0,                         /* tp_weaklistoffset */
  0,                         /* tp_iter */
  0,                         /* tp_iternext */
  </xsl:text> <xsl:value-of select="$name"/>
        <xsl:text>_methods,  /* tp_methods */
  </xsl:text> <xsl:value-of select="$name"/>
        <xsl:text>_members,  /* tp_members */
  0,                         /* tp_getset */
  0,                         /* tp_base */
  0,                         /* tp_dict */
  0,                         /* tp_descr_get */
  0,                         /* tp_descr_set */
  0,                         /* tp_dictoffset */
  (initproc)</xsl:text> <xsl:value-of select="$name"/>
           <xsl:text>_init,  /* tp_init */
  0,                         /* tp_alloc */
  </xsl:text>  <xsl:value-of select="$name"/>
            <xsl:text>_new,  /* tp_new */</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>};</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>
</xsl:template>	


<!--
  - definePyModuleInit
  -->
<xsl:template name="definePyModuleInit">
  <xsl:param name="name" select="@name"/>
  <xsl:text>#ifndef PyMODINIT_FUNC</xsl:text>
  <xsl:text>/* declarations for DLL import/export */</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>#define PyMODINIT_FUNC void</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>#endif</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>PyMODINIT_FUNC</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>init</xsl:text> <xsl:value-of select="$name"/>
  <xsl:text>(void)</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>{</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>  PyObject* m;</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>
  <xsl:text>  if (PyType_Ready(&amp;</xsl:text> <xsl:value-of select="$name"/>
  <xsl:text>Type) &lt; 0) {</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>    return;</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>  }</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>  m = Py_InitModule3(&quot;</xsl:text>
  <xsl:value-of select="$name"/>
  <xsl:text>&quot;, </xsl:text> <xsl:value-of select="$name"/>
  <xsl:text>_methods,</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>                     "CCA </xsl:text>
  <xsl:value-of select="$name"/> <xsl:text> component module.");</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>
  <xsl:text>  if (m == NULL) return;</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>
  <xsl:text>  Py_INCREF(&amp;</xsl:text> <xsl:value-of select="$name"/>
  <xsl:text>Type);</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:text>  PyModule_AddObject(m, &quot;</xsl:text>
  <xsl:value-of select="$name"/> <xsl:text>&quot;,</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>                        (PyObject*) &amp;</xsl:text>
  <xsl:value-of select="$name"/> <xsl:text>Type);</xsl:text>
  <xsl:value-of select="$nl"/>
  <xsl:text>}</xsl:text> <xsl:value-of select="$nl"/>
  <xsl:value-of select="$nl"/>
</xsl:template>	


</xsl:stylesheet>
