<!-- LANL:license
 ...........................................................................
 - This SOFTWARE has been authored by an employee or employees of the
 - University of California, operator of the Los Alamos National Laboratory
 - under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 - The U.S. Government has rights to use, reproduce, and distribute this
 - SOFTWARE.  The public may copy, distribute, prepare derivative works and
 - publicly display this SOFTWARE without charge, provided that this Notice
 - and any statement of authorship are reproduced on all copies.  Neither
 - the Government nor the University makes any warranty, express or implied,
 - or assumes any liability or responsibility for the use of this SOFTWARE.
 - If SOFTWARE is modified to produce derivative works, such modified
 - SOFTWARE should be clearly marked, so as not to confuse it with the
 - version available from LANL.
 ...........................................................................
 - LANL:license
 ...........................................................................
 -->

<!--
  - common templates for Chasm language interoperability system
  -
  - variables:
  -   nl - a newline "\n"
  -
  - templates:
  -
  - include-files
  - include-file (filename)
  - include-file-local (filename)
  -
  - upper-case (symbol)
  - lower-case (symbol)
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

<xsl:output method="text"/>


<!--
  - nl: a newline "\n"
  -->
<xsl:variable name="nl">
<xsl:text>
</xsl:text>
</xsl:variable>


<!--
  - include-files: include header files (to be overridden, this is a noop)
  -->
<xsl:template name="include-files"/>


<!--
  - include-file (filename): include named header file
  -->
<xsl:template name="include-file">
  <xsl:param name="filename"/>
  <xsl:text>#include &lt;</xsl:text> <xsl:value-of select="$filename"/>
  <xsl:text>&gt;</xsl:text> <xsl:value-of select="$nl"/>
</xsl:template>


<!--
  - include-file-local (filename): include named header file
  -->
<xsl:template name="include-file-local">
  <xsl:param name="filename"/>
  <xsl:text>#include &quot;</xsl:text> <xsl:value-of select="$filename"/>
  <xsl:text>&quot;</xsl:text> <xsl:value-of select="$nl"/>
</xsl:template>


<!--
  - upper-case (symbol): output symbol in upper case
  -->
<xsl:template name="upper-case">
  <xsl:param name="symbol"/>
  <xsl:value-of select="translate($symbol,
                                  'abcdefghijklmnopqrstuvwxyz',
                                  'ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
</xsl:template>


<!--
  - lower-case (symbol): output symbol in lower case
  -->
<xsl:template name="lower-case">
  <xsl:param name="symbol"/>
  <xsl:value-of select="translate($symbol,
                                  'ABCDEFGHIJKLMNOPQRSTUVWXYZ',
                                  'abcdefghijklmnopqrstuvwxyz')"/>
</xsl:template>


<!--
  - name-ifnot-alias : choose alias or name if no alias
  -->
<xsl:template name="name-ifnot-alias">
  <xsl:choose>
    <xsl:when test="@alias">
      <xsl:value-of select="@alias"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="@name"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


</xsl:stylesheet>
