<!-- LANL:license
 ...........................................................................
 - This SOFTWARE has been authored by an employee or employees of the
 - University of California, operator of the Los Alamos National Laboratory
 - under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 - The U.S. Government has rights to use, reproduce, and distribute this
 - SOFTWARE.  The public may copy, distribute, prepare derivative works and
 - publicly display this SOFTWARE without charge, provided that this Notice
 - and any statement of authorship are reproduced on all copies.  Neither
 - the Government nor the University makes any warranty, express or implied,
 - or assumes any liability or responsibility for the use of this SOFTWARE.
 - If SOFTWARE is modified to produce derivative works, such modified
 - SOFTWARE should be clearly marked, so as not to confuse it with the
 - version available from LANL.
 ...........................................................................
 - LANL:license
 ...........................................................................
 -->

<!--
  - templates for type conversions
  -
  - variables:
  -
  - templates:
  -
  - type-qual-rtn
  - type-spec
  -
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

<xsl:output method="text"/>


<!--
  - type-spec
  -->
<xsl:template name="type-spec">
  <xsl:choose>
    <xsl:when test="@kind = 'fvoid'">
      <xsl:text>void</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'fint'">
      <xsl:choose>
        <xsl:when test="@ikind = 'char'">
          <xsl:text>char</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'uchar'">
          <xsl:text>unsigned char</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'short'">
          <xsl:text>short</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'ushort'">
          <xsl:text>unsigned short</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'int'">
          <xsl:text>int</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'uint'">
          <xsl:text>unsigned int</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'long'">
          <xsl:text>long</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'ulong'">
          <xsl:text>unsigned long</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'longlong'">
          <xsl:text>long long</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'ulonglong'">
          <xsl:text>unsigned long long</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>UNSUPPORTED</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:when test="@kind = 'ffloat'">
      <xsl:choose>
        <xsl:when test="@fkind = 'float'">
          <xsl:text>float</xsl:text>
        </xsl:when>
        <xsl:when test="@fkind = 'dbl'">
          <xsl:text>double</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>UNSUPPORTED</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:when test="@kind = 'flogic'">
      <xsl:text>UNSUPPORTED</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'fchar'">
      <xsl:text>UNSUPPORTED</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'farray'">
      <xsl:text>UNSUPPORTED</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'usertype'">
      <xsl:text>UNSUPPORTED</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'fptr'">
      <xsl:text>UNSUPPORTED</xsl:text>
    </xsl:when>
    <xsl:otherwise>
      <xsl:text>UNSUPPORTED</xsl:text>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!--
  - rtn-id <return/type>
  -->
<xsl:template name="rtn-id">
  <xsl:param name="depth" select="0"/>
  <xsl:choose>

    <!-- C++ types -->

    <xsl:when test="@kind = 'void'"/>
    <xsl:when test="@kind = 'int'"/>
    <xsl:when test="@kind = 'float'"/>
    <xsl:when test="@kind = 'ptr'">
      <xsl:text>rtn_</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'ref'">
      <xsl:for-each select="indirect/type">
        <xsl:call-template name="rtn-id">
          <xsl:with-param name="depth" select="../@depth"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:when>
    <xsl:when test="@kind = 'usertype'">
      <xsl:choose>
        <xsl:when test="@usertype = 'std::string'"/>
        <xsl:otherwise>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>

    <!-- Fortran types -->

    <xsl:when test="@kind = 'fvoid'"/>
    <xsl:when test="@kind = 'fint'">
      <xsl:text>rtn_</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'ffloat'">
      <xsl:text>rtn_</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'flogic'">
      <xsl:text>rtn_</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'fchar'"/>
    <xsl:when test="@kind = 'farray'"/>
    <xsl:when test="@kind = 'usertype'"/>
    <xsl:when test="@kind = 'fptr'"/>
    <xsl:otherwise>
      <xsl:text>UNSUPPORTED</xsl:text>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!--
  - assignment-operator <return/type>
  -->
<xsl:template name="assignment-operator">
  <xsl:param name="depth" select="0"/>
  <xsl:choose>

    <!-- C++ types -->

    <xsl:when test="@kind = 'void'"/>
    <xsl:when test="@kind = 'int'">
      <xsl:text>return </xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'float'">
      <xsl:text>return </xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'ptr'">
        <xsl:text> = </xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'ref'">
      <xsl:for-each select="indirect/type">
        <xsl:call-template name="assignment-operator">
          <xsl:with-param name="depth" select="../@depth"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:when>
    <xsl:when test="@kind = 'usertype'">
      <xsl:choose>
        <xsl:when test="@usertype = 'std::string'">
          <xsl:text>return </xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>return </xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>

    <!-- Fortran types -->

    <xsl:when test="@kind = 'fvoid'"/>
    <xsl:when test="@kind = 'fint'">
      <xsl:text> = </xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'ffloat'">
      <xsl:text> = </xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'flogic'">
      <xsl:text> = </xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'fchar'"/>
    <xsl:when test="@kind = 'farray'"/>
    <xsl:when test="@kind = 'usertype'"/>
    <xsl:when test="@kind = 'fptr'"/>
    <xsl:otherwise>
      <xsl:text>UNSUPPORTED</xsl:text>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


</xsl:stylesheet>
