<!-- LANL:license
 ...........................................................................
 - This SOFTWARE has been authored by an employee or employees of the
 - University of California, operator of the Los Alamos National Laboratory
 - under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 - The U.S. Government has rights to use, reproduce, and distribute this
 - SOFTWARE.  The public may copy, distribute, prepare derivative works and
 - publicly display this SOFTWARE without charge, provided that this Notice
 - and any statement of authorship are reproduced on all copies.  Neither
 - the Government nor the University makes any warranty, express or implied,
 - or assumes any liability or responsibility for the use of this SOFTWARE.
 - If SOFTWARE is modified to produce derivative works, such modified
 - SOFTWARE should be clearly marked, so as not to confuse it with the
 - version available from LANL.
 ...........................................................................
 - LANL:license
 ...........................................................................
 -->

<!--
  - templates for type conversions
  -
  - variables:
  -
  - templates:
  -
  - type-spec-py
  - type-qual-rtn  
  - type-spec-rtn   
  -
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

<xsl:output method="text"/>


<!--
  - type-spec-py
  -->
<xsl:template name="type-spec-py">
  <xsl:param name="depth" select="0"/>
  <xsl:choose>

    <!-- C++ types -->

    <xsl:when test="@kind = 'void'">
      <xsl:text>void</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'ptr'">
      <xsl:for-each select="indirect/type">
        <xsl:call-template name="type-spec-ptr-py">
          <xsl:with-param name="depth" select="../@depth"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:when>
    <xsl:when test="@kind = 'ref'">
      <xsl:for-each select="indirect/type">
        <xsl:call-template name="type-spec-py">
          <xsl:with-param name="depth" select="../@depth"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:when>
    <xsl:when test="@kind = 'usertype'">
      <xsl:choose>
        <xsl:when test="@usertype = 'std::string'">
          <xsl:text>char*</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="@usertype"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>

    <!-- Fortran types -->

    <xsl:when test="@kind = 'fvoid'">
      <xsl:text>void</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'fint'">
      <xsl:choose>
        <xsl:when test="@ikind = 'char'">
          <xsl:text>char</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'uchar'">
          <xsl:text>unsigned char</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'short'">
          <xsl:text>short</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'ushort'">
          <xsl:text>unsigned short</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'int'">
          <xsl:text>int</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'uint'">
          <xsl:text>unsigned int</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'long'">
          <xsl:text>long</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'ulong'">
          <xsl:text>unsigned long</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'longlong'">
          <xsl:text>long long</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'ulonglong'">
          <xsl:text>unsigned long long</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>UNSUPPORTED</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:when test="@kind = 'ffloat'">
      <xsl:choose>
        <xsl:when test="@fkind = 'float'">
          <xsl:text>float</xsl:text>
        </xsl:when>
        <xsl:when test="@fkind = 'dbl'">
          <xsl:text>double</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>UNSUPPORTED</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:when test="@kind = 'fcmplx'">
      <xsl:text>Py_complex</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'flogic'">
      <xsl:text>UNSUPPORTED</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'fchar'">
      <xsl:text>char*</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'farray'">
      <xsl:text>UNSUPPORTED</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'usertype'">
      <xsl:text>UNSUPPORTED</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'fptr'">
      <xsl:text>UNSUPPORTED</xsl:text>
    </xsl:when>
    <xsl:otherwise>
      <xsl:text>UNSUPPORTED</xsl:text>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!--
  - type-spec-ptr-py <type> - is a pointer
  -->
<xsl:template name="type-spec-ptr-py">
  <xsl:param name="depth" select="0"/>
  <xsl:choose>

    <!-- C++ types -->

    <xsl:when test="@kind = 'usertype'">
      <xsl:text>PyCObject*</xsl:text>
    </xsl:when>
    <xsl:otherwise>
      <xsl:text>UNSUPPORTED</xsl:text>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!--
  - type-qual-rtn
  -->
<xsl:template name="type-qual-rtn">
  <xsl:text>static </xsl:text>
</xsl:template>


<!--
  - type-spec-rtn
  -->
<xsl:template name="type-spec-rtn">
  <xsl:text>PyObject*</xsl:text>
</xsl:template>


<!--
  - type-conv-str
  -->
<xsl:template name="type-conv-str">
  <xsl:param name="depth" select="0"/>
  <xsl:choose>

    <!-- C++ types -->

    <xsl:when test="@kind = 'void'"/>
    <xsl:when test="@kind = 'ptr'">
      <xsl:text>O</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'ref'">
      <xsl:for-each select="indirect/type">
        <xsl:call-template name="type-conv-str">
          <xsl:with-param name="depth" select="../@depth"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:when>
    <xsl:when test="@kind = 'usertype'">
      <xsl:choose>
        <xsl:when test="@usertype = 'std::string'">
          <xsl:text>s</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>O&amp;</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>

    <!-- Fortran types -->

    <xsl:when test="@kind = 'fvoid'"/>
    <xsl:when test="@kind = 'fint'">
      <xsl:choose>
        <xsl:when test="@ikind = 'char'">
          <xsl:text>B</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'uchar'">
          <xsl:text>B</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'short'">
          <xsl:text>h</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'ushort'">
          <xsl:text>h</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'int'">
          <xsl:text>i</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'uint'">
          <xsl:text>i</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'long'">
          <xsl:text>l</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'ulong'">
          <xsl:text>l</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'longlong'">
          <xsl:text>UNSUPPORTED</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'ulonglong'">
          <xsl:text>UNSUPPORTED</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>UNSUPPORTED</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:when test="@kind = 'ffloat'">
      <xsl:choose>
        <xsl:when test="@fkind = 'float'">
          <xsl:text>f</xsl:text>
        </xsl:when>
        <xsl:when test="@fkind = 'dbl'">
          <xsl:text>d</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>UNSUPPORTED</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:when test="@kind = 'fcmplx'">
      <xsl:text>D</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'flogic'">
      <xsl:text>UNSUPPORTED</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'fchar'">
      <xsl:text>s</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'farray'">
      <xsl:text>UNSUPPORTED</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'usertype'">
      <xsl:text>UNSUPPORTED</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'fptr'">
      <xsl:text>UNSUPPORTED</xsl:text>
    </xsl:when>
    <xsl:otherwise>
      <xsl:text>UNSUPPORTED</xsl:text>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<!--
  - type-conv-str-rtn
  -->
<xsl:template name="type-conv-str-rtn">
  <xsl:param name="depth" select="0"/>

  <!-- Fortran types -->
  <xsl:if test="../@intent='out'">
    <xsl:choose>
      <xsl:when test="@kind = 'fvoid'"/>
      <xsl:when test="@kind = 'fint'">
        <xsl:choose>
          <xsl:when test="@ikind = 'char'">
            <xsl:text>B</xsl:text>
          </xsl:when>
          <xsl:when test="@ikind = 'uchar'">
            <xsl:text>B</xsl:text>
          </xsl:when>
          <xsl:when test="@ikind = 'short'">
            <xsl:text>h</xsl:text>
          </xsl:when>
          <xsl:when test="@ikind = 'ushort'">
            <xsl:text>h</xsl:text>
          </xsl:when>
          <xsl:when test="@ikind = 'int'">
            <xsl:text>i</xsl:text>
          </xsl:when>
          <xsl:when test="@ikind = 'uint'">
            <xsl:text>i</xsl:text>
          </xsl:when>
          <xsl:when test="@ikind = 'long'">
            <xsl:text>l</xsl:text>
          </xsl:when>
          <xsl:when test="@ikind = 'ulong'">
            <xsl:text>l</xsl:text>
          </xsl:when>
          <xsl:when test="@ikind = 'longlong'">
            <xsl:text>UNSUPPORTED</xsl:text>
          </xsl:when>
          <xsl:when test="@ikind = 'ulonglong'">
            <xsl:text>UNSUPPORTED</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>UNSUPPORTED</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="@kind = 'ffloat'">
        <xsl:choose>
          <xsl:when test="@fkind = 'float'">
            <xsl:text>f</xsl:text>
          </xsl:when>
          <xsl:when test="@fkind = 'dbl'">
            <xsl:text>d</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>UNSUPPORTED</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="@kind = 'flogic'">
        <xsl:text>UNSUPPORTED</xsl:text>
      </xsl:when>
      <xsl:when test="@kind = 'fchar'">
        <xsl:text>s</xsl:text>
      </xsl:when>
      <xsl:when test="@kind = 'farray'">
        <xsl:text>UNSUPPORTED</xsl:text>
      </xsl:when>
      <xsl:when test="@kind = 'usertype'">
        <xsl:text>UNSUPPORTED</xsl:text>
      </xsl:when>
      <xsl:when test="@kind = 'fptr'">
        <xsl:text>UNSUPPORTED</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>UNSUPPORTED</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:if>
</xsl:template>


<!--
  - arg-id-conv-ParseTuple
  -->
<xsl:template name="arg-id-conv-ParseTuple">
  <xsl:param name="depth" select="0"/>
  <xsl:choose>

    <!-- C++ types -->
    <xsl:when test="@kind = 'ref'">
      <xsl:for-each select="indirect/type">
        <xsl:call-template name="arg-id-conv-ParseTuple">
          <xsl:with-param name="depth" select="../@depth"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:when>
    <xsl:when test="@kind = 'usertype'">
      <xsl:choose>
        <xsl:when test="@usertype = 'std::string'"/>
        <xsl:otherwise>
          <xsl:value-of select="@usertype"/>
          <xsl:text>_FromPy, </xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>

    <!-- Fortran types -->
    <xsl:when test="@kind = 'usertype'">
      <xsl:value-of select="@usertype"/>
      <xsl:text>_FromPy, </xsl:text>
    </xsl:when>
  </xsl:choose>
</xsl:template>


<!--
  - rtn-id-conv-ParseTuple
  -->
<xsl:template name="rtn-id-conv-ParseTuple">
  <xsl:param name="depth" select="0"/>
  <xsl:choose>

    <!-- C++ types -->
    <xsl:when test="@kind = 'ref'">
      <xsl:for-each select="indirect/type">
        <xsl:call-template name="rtn-id-conv-ParseTuple">
          <xsl:with-param name="depth" select="../@depth"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:when>
    <xsl:when test="@kind = 'usertype'">
      <xsl:choose>
        <xsl:when test="@usertype = 'std::string'"/>
        <xsl:otherwise>
          <xsl:value-of select="@usertype"/>
          <xsl:text>_ToPy, </xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>

    <!-- Fortran types -->
    <xsl:when test="@kind = 'usertype'">
      <xsl:value-of select="@usertype"/>
      <xsl:text>_ToPy, </xsl:text>
    </xsl:when>
  </xsl:choose>
</xsl:template>


<!--
  - address-operator-ParseTuple
  -->
<xsl:template name="address-operator-ParseTuple">
  <xsl:param name="depth" select="0"/>
  <xsl:choose>
    <!-- C++ types -->
    <xsl:when test="@kind = 'void'"/>
    <!-- Fortran types -->
    <xsl:when test="@kind = 'fvoid'"/>
    <xsl:when test="@kind = 'fptr'"/>
    <xsl:otherwise> <xsl:text>&amp;</xsl:text> </xsl:otherwise>
  </xsl:choose>
</xsl:template>


</xsl:stylesheet>
