<!--
  - templates for sidl type conversions
  -
  - variables:
  -
  - templates:
  -   type-qual   
  -   type-spec   
  -
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

<xsl:import href="type-conv-user.xsl"/>

<xsl:output method="text"/>

<!--
  - type-qual <type>
  -   override common-C to add intent qualifier
  -->
<xsl:template name="type-qual">
  <xsl:value-of select="@intent"/>
</xsl:template>


<!--
  - type-spec <type>
  -   override common-C to output sidl types
  -->
<xsl:template name="type-spec">
  <xsl:param name="depth" select="0"/>
  <xsl:variable name="amp"/>

  <xsl:variable name="out_text">

  <xsl:choose>

    <!-- C++ types -->

    <xsl:when test="@kind = 'void'">
      <xsl:text>void</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'int'">
      <xsl:choose>
        <xsl:when test="@ikind = 'char'">
          <xsl:text>char</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'int'">
          <xsl:text>int</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'long'">
          <xsl:text>long</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>NOT_A_BABEL_TYPE</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:when test="@kind = 'float'">
      <xsl:choose>
        <xsl:when test="@fkind = 'float'">
          <xsl:text>float</xsl:text>
        </xsl:when>
        <xsl:when test="@fkind = 'dbl'">
          <xsl:text>double</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>NOT_A_BABEL_TYPE</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:when test="@kind = 'ptr'">
      <xsl:for-each select="indirect/type">
        <xsl:choose>
          <xsl:when test="@kind = 'void'">
            <xsl:text>opaque</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>NOT_A_BABEL_TYPE</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:for-each>
    </xsl:when>
    <xsl:when test="@kind = 'ref'">
      <xsl:for-each select="indirect/type">
        <xsl:call-template name="type-spec">
          <xsl:with-param name="depth" select="../@depth"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:when>
    <xsl:when test="@kind = 'usertype'">
      <xsl:choose>
        <xsl:when test="@usertype = 'std::string'">
          <xsl:text>string</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>NOT_A_BABEL_TYPE</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>

    <!-- Fortran types -->

    <xsl:when test="@kind = 'fvoid'">
      <xsl:text>void</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'fint'">
      <xsl:choose>
        <xsl:when test="@ikind = 'char'">
          <xsl:text>char</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'int'">
          <xsl:text>int</xsl:text>
        </xsl:when>
        <xsl:when test="@ikind = 'long'">
          <xsl:text>long</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>NOT_A_BABEL_TYPE</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:when test="@kind = 'ffloat'">
      <xsl:choose>
        <xsl:when test="@fkind = 'float'">
          <xsl:text>float</xsl:text>
        </xsl:when>
        <xsl:when test="@fkind = 'dbl'">
          <xsl:text>double</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>NOT_A_BABEL_TYPE</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:when test="@kind = 'fcmplx'">
      <xsl:choose>
        <xsl:when test="@fkind = 'float'">
          <xsl:text>fcomplex</xsl:text>
        </xsl:when>
        <xsl:when test="@fkind = 'dbl'">
          <xsl:text>dcomplex</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>NOT_A_BABEL_TYPE</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:when test="@kind = 'flogic'">
      <xsl:text>bool</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'fchar'">
      <xsl:text>string</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'farray'">
      <xsl:text>array&lt;</xsl:text>
        <xsl:for-each select="array/type">
          <xsl:call-template name="type-spec"/>
        </xsl:for-each>
      <xsl:value-of select="concat(',', array/@rank, '&gt;')"/>
    </xsl:when>
    <xsl:when test="@kind = 'usertype'">
      <xsl:text>opaque</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'fptr'">
      <xsl:for-each select="type/indirect">
        <xsl:call-template name="type-spec">
          <xsl:with-param name="depth" select="../@depth"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:when>
    <xsl:otherwise>
      <xsl:text>NOT_A_BABEL_TYPE</xsl:text>
    </xsl:otherwise>
  </xsl:choose>

  </xsl:variable>

  <xsl:choose>
    <xsl:when test="$out_text = 'NOT_A_BABEL_TYPE'">
      <xsl:call-template name="type-spec-user">
        <xsl:with-param name="out_default" select="$out_text"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$out_text"/>
    </xsl:otherwise>
  </xsl:choose>

</xsl:template>


<!--
  - pointer
  -   override common-C to output sidl types (no * needed for sidl)
  -->
<xsl:template name="pointer"/>


<!--
  - rtn-id <return/type>
  -   override type-conversions to return id (Babel uses retval)
  -->
<xsl:template name="rtn-id">
  <xsl:param name="depth" select="0"/>
  <xsl:choose>

    <!-- Fortran types -->

    <xsl:when test="@kind = 'fvoid'"/>
    <xsl:when test="@kind = 'flogic'">
      <xsl:text>retval</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'fint'">
      <xsl:text>retval</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'ffloat'">
      <xsl:text>retval</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'fcmplx'">
      <xsl:text>retval</xsl:text>
    </xsl:when>
    <xsl:when test="@kind = 'fchar'"/>
    <xsl:when test="@kind = 'farray'"/>
    <xsl:when test="@kind = 'usertype'"/>
    <xsl:when test="@kind = 'fptr'"/>
    <xsl:otherwise>
      <xsl:text>UNSUPPORTED</xsl:text>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


</xsl:stylesheet>
