<!-- LANL:license
 ...........................................................................
 - This SOFTWARE has been authored by an employee or employees of the
 - University of California, operator of the Los Alamos National Laboratory
 - under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 - The U.S. Government has rights to use, reproduce, and distribute this
 - SOFTWARE.  The public may copy, distribute, prepare derivative works and
 - publicly display this SOFTWARE without charge, provided that this Notice
 - and any statement of authorship are reproduced on all copies.  Neither
 - the Government nor the University makes any warranty, express or implied,
 - or assumes any liability or responsibility for the use of this SOFTWARE.
 - If SOFTWARE is modified to produce derivative works, such modified
 - SOFTWARE should be clearly marked, so as not to confuse it with the
 - version available from LANL.
 ...........................................................................
 - LANL:license
 ...........................................................................
 -->

<!--
  - chasm_call.cc.xsl: creates a file of C functions that call Fortran
  -   module procedures.
  -
  - Output should be directed to the file, MODULE_call.cc
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

  <xsl:import href="name-conversions.xsl"/>
  <xsl:import href="type-conversions.xsl"/>

  <xsl:output method="text"/>

  <xsl:variable name="newline">
<xsl:text>
</xsl:text>
  </xsl:variable>

 <!--
    - root level
    -->
  <xsl:template match="/">
    <xsl:apply-templates select="library"/>
  </xsl:template>


  <!--
    - library level
    -->
  <xsl:template match="library">

    <xsl:call-template name="openFile"/>
    <xsl:apply-templates select="scope" mode="defineF90Symbols"/>
    <xsl:apply-templates select="scope" mode="declareGlobalFunctions"/>
    <xsl:apply-templates select="scope" mode="declareFunctionPointers"/>
    <xsl:apply-templates select="scope" mode="defineSetFunctionPointers"/>
    <xsl:apply-templates select="scope" mode="defineBridgingFunctions"/>
    <xsl:call-template name="closeFile"/>

  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="scope" mode="defineF90Symbols">
    <xsl:if test="method">
      <xsl:text>#if defined(F90_SYM_CASE_LOWER)</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:if test="method/@kind">
        <xsl:apply-templates select="method" mode="define-F90-symbols-lower"/>
      </xsl:if>
      <xsl:text>#endif /* F90_SYM_CASE_LOWER) */</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:value-of select="$newline"/>

      <xsl:text>#if defined(F90_SYM_CASE_MIXED)</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:if test="method/@kind">
        <xsl:apply-templates select="method" mode="define-F90-symbols-mixed"/>
      </xsl:if>
      <xsl:text>#endif /* F90_SYM_CASE_MIXED) */</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:value-of select="$newline"/>

      <xsl:text>#if defined(F90_SYM_CASE_UPPER)</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:if test="method/@kind">
        <xsl:apply-templates select="method" mode="define-F90-symbols-upper"/>
      </xsl:if>
      <xsl:text>#endif /* F90_SYM_CASE_UPPER) */</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:value-of select="$newline"/>
    </xsl:if>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="scope" mode="declareGlobalFunctions">

    <xsl:if test="@kind = 'global'">
      <xsl:if test="method">
        <xsl:if test="position() = '1'">
          <xsl:text>//</xsl:text>
          <xsl:value-of select="$newline"/>
          <xsl:text>// Global Fortran procedure prototypes</xsl:text>
          <xsl:value-of select="$newline"/>
          <xsl:text>//</xsl:text>
          <xsl:value-of select="$newline"/>
          <xsl:value-of select="$newline"/>
        </xsl:if>
        <xsl:call-template name="openExternC"/>

        <xsl:if test="method/@kind">
          <xsl:apply-templates select="method" mode="declareGlobalFunction"/>
        </xsl:if>

        <xsl:call-template name="closeExternC"/>
        <xsl:value-of select="$newline"/>
      </xsl:if>
    </xsl:if>

  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="scope" mode="declareFunctionPointers">

    <xsl:if test="method">
      <xsl:if test="position() = '1'">
        <xsl:text>//</xsl:text>
        <xsl:value-of select="$newline"/>
        <xsl:text>// Pointers to Fortran procedures</xsl:text>
        <xsl:value-of select="$newline"/>
        <xsl:text>//</xsl:text>
        <xsl:value-of select="$newline"/>
        <xsl:value-of select="$newline"/>
      </xsl:if>

      <xsl:if test="method/@kind">
        <xsl:apply-templates select="method" mode="declareFunctionPointers"/>
      </xsl:if>

      <xsl:value-of select="$newline"/>
    </xsl:if>

  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="scope" mode="defineSetFunctionPointers">
    <xsl:if test="@kind = 'module'">
      <xsl:if test="method">
        <xsl:if test="position() = '1'">
          <xsl:text>//</xsl:text>
          <xsl:value-of select="$newline"/>
          <xsl:text>// Functions (called from Fortran) to set the pointers to Fortran procedures</xsl:text>
          <xsl:value-of select="$newline"/>
          <xsl:text>//</xsl:text>
          <xsl:value-of select="$newline"/>
          <xsl:value-of select="$newline"/>
        </xsl:if>

        <xsl:call-template name="openExternC"/>
        <xsl:if test="method/@kind">
          <xsl:apply-templates select="method" mode="defineFunctionPointers"/>
        </xsl:if>

        <xsl:value-of select="$newline"/>
        <xsl:call-template name="closeExternC"/>
        <xsl:value-of select="$newline"/>
        <xsl:value-of select="$newline"/>
      </xsl:if>
    </xsl:if>

  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="scope" mode="defineBridgingFunctions">

    <xsl:if test="method">
      <xsl:if test="position() = '1'">
        <xsl:text>//</xsl:text>
        <xsl:value-of select="$newline"/>
        <xsl:text>// C Bridging functions to call Fortran procedures</xsl:text>
        <xsl:value-of select="$newline"/>
        <xsl:text>//</xsl:text>
        <xsl:value-of select="$newline"/>
      </xsl:if>

      <xsl:if test="method/@kind">
        <xsl:apply-templates select="method" mode="define-CtoF-calls"/>
      </xsl:if>

    </xsl:if>

  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="method" mode="define-F90-symbols-lower">
    <xsl:variable name="symbol">
      <xsl:text>SET_</xsl:text>
      <xsl:call-template name="methodName"/>
    </xsl:variable>
    <xsl:if test="@kind">
      <xsl:call-template name="define-F90-symbol-lower">
        <xsl:with-param name="symbol" select="$symbol"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="method" mode="define-F90-symbols-mixed">
    <xsl:variable name="symbol">
      <xsl:text>SET_</xsl:text>
      <xsl:call-template name="methodName"/>
    </xsl:variable>
    <xsl:if test="@kind">
      <xsl:call-template name="define-F90-symbol-mixed">
        <xsl:with-param name="symbol" select="$symbol"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="method" mode="define-F90-symbols-upper">
    <xsl:variable name="symbol">
      <xsl:text>SET_</xsl:text>
      <xsl:call-template name="methodName"/>
    </xsl:variable>
    <xsl:if test="@kind">
      <xsl:call-template name="define-F90-symbol-upper">
        <xsl:with-param name="symbol" select="$symbol"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="method" mode="declareFunctionPointers">
    <xsl:variable name="rtn_pre">
      <xsl:apply-templates select="return" mode="decl_cf_rtn_pre"/>
    </xsl:variable>

    <xsl:if test="@kind">
      <xsl:apply-templates select="return" mode="decl_cf_rtn"/>
      <xsl:text> (*</xsl:text>
      <xsl:call-template name="methodName"/>
      <xsl:text>)(</xsl:text>
      <xsl:apply-templates select="return" mode="decl_cf_rtn_pre"/>
      <xsl:apply-templates select="arg" mode="decl_cf_arg"/>
      <xsl:apply-templates select="return" mode="decl_cf_rtn_post"/>
      <xsl:apply-templates select="arg" mode="decl_cf_rtn_post"/>
      <xsl:text>)</xsl:text>
      <xsl:if test="../@kind = 'global'">
        <xsl:text> = </xsl:text>
        <xsl:if test="$rtn_pre != ''">
          <xsl:text>F_GBL_</xsl:text>
        </xsl:if>
        <xsl:value-of select="@name"/>
      </xsl:if>
      <xsl:text>;</xsl:text>
      <xsl:value-of select="$newline"/>
    </xsl:if>

  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="method" mode="declareGlobalFunction">
    <xsl:variable name="rtn_pre">
      <xsl:apply-templates select="return" mode="decl_cf_rtn_pre"/>
    </xsl:variable>

    <xsl:if test="@kind">
      <xsl:text>  </xsl:text>
      <xsl:apply-templates select="return" mode="decl_cf_rtn"/>
      <xsl:text> </xsl:text>
      <xsl:if test="$rtn_pre != ''">
        <xsl:text>F_GBL_</xsl:text>
      </xsl:if>

      <xsl:value-of select="@name"/>
      <xsl:text>(</xsl:text>
      <xsl:apply-templates select="return" mode="decl_cf_rtn_pre"/>
      <xsl:apply-templates select="arg" mode="decl_cf_arg"/>
      <xsl:apply-templates select="return" mode="decl_cf_rtn_post"/>
      <xsl:apply-templates select="arg" mode="decl_cf_rtn_post"/>
      <xsl:text>)</xsl:text>
      <xsl:text>;</xsl:text>
      <xsl:value-of select="$newline"/>
    </xsl:if>

  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="method" mode="defineFunctionPointers">

    <xsl:if test="@kind">
      <xsl:variable name="procedure"
           select="concat('SET_', ../@name, '_', @name)"/>

      <xsl:value-of select="$newline"/>
      <xsl:text>void </xsl:text>
      <xsl:call-template name="CSymbolName">
        <xsl:with-param name="symbol" select="$procedure"/>
      </xsl:call-template>
      <xsl:text>( </xsl:text>
      <xsl:apply-templates select="return" mode="decl_cf_rtn"/>
      <xsl:text> (*fptr)(</xsl:text>
      <xsl:apply-templates select="return" mode="decl_cf_rtn_pre"/>
      <xsl:apply-templates select="arg" mode="decl_cf_arg"/>
      <xsl:apply-templates select="return" mode="decl_cf_rtn_post"/>
      <xsl:text>) )</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:text>{</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:text>  </xsl:text>
      <xsl:call-template name="methodName"/>
      <xsl:text> = fptr;</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:text>}</xsl:text>
      <xsl:value-of select="$newline"/>
    </xsl:if>

  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="method" mode="define-CtoF-calls">
    <xsl:variable name="rtn_pre">
      <xsl:apply-templates select="return" mode="decl_cc_rtn_pre"/>
    </xsl:variable>
    <xsl:variable name="call_rtn_pre">
      <xsl:apply-templates select="return" mode="call_cf_rtn_pre"/>
    </xsl:variable>

    <xsl:if test="@kind">
      <xsl:value-of select="$newline"/>
      <xsl:apply-templates select="return" mode="decl_cc_rtn"/>
      <xsl:text> </xsl:text>
      <xsl:value-of select="../@name"/>
      <xsl:text>::</xsl:text>
      <xsl:value-of select="@name"/>
      <xsl:text>(</xsl:text>

      <xsl:if test="$rtn_pre != ''">
        <xsl:value-of select="concat($rtn_pre, ' chasm_rtn')"/>
        <xsl:if test="arg">
          <xsl:text>, </xsl:text>
        </xsl:if>
      </xsl:if>

      <xsl:apply-templates select="arg" mode="decl_cc_arg"/>
      <xsl:text>)</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:text>{</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:apply-templates select="return" mode="decl_call_cf_rtn"/>
      <xsl:text>  </xsl:text>
      <xsl:apply-templates select="return" mode="call_cf_rtn"/>
      <xsl:call-template name="methodName"/>
      <xsl:text>(</xsl:text>

      <xsl:if test="$call_rtn_pre != ''">
        <xsl:value-of select="$call_rtn_pre"/>
        <xsl:if test="arg">
          <xsl:text>, </xsl:text>
        </xsl:if>
      </xsl:if>

      <xsl:apply-templates select="arg" mode="call_cf"/>
      <xsl:apply-templates select="return" mode="call_cf_rtn_post"/>
      <xsl:apply-templates select="arg" mode="call_cf_arg_str_post"/>
      <xsl:text>);</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:apply-templates select="return" mode="assign_call_cf_rtn"/>
      <xsl:text>}</xsl:text>
      <xsl:value-of select="$newline"/>
    </xsl:if>

  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="return" mode="decl_cf_rtn">
    <xsl:call-template name="decl_cf_rtn"/>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="return" mode="decl_cf_rtn_pre">
    <xsl:variable name="rtn_pre">
      <xsl:call-template name="decl_cf_rtn_pre"/>
    </xsl:variable>

    <xsl:if test="$rtn_pre != ''">
      <xsl:value-of select="$rtn_pre"/>
      <xsl:if test="../arg">
        <xsl:text>, </xsl:text>
      </xsl:if>
    </xsl:if>

  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="return" mode="decl_cf_rtn_post">
    <xsl:call-template name="decl_cf_rtn_post"/>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="arg" mode="decl_cf_rtn_post">
    <xsl:call-template name="decl_cf_rtn_post"/>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="arg" mode="decl_cf_arg">

    <xsl:call-template name="decl_cf_arg"/>

    <xsl:if test="position() != last()">
      <xsl:text>, </xsl:text>
    </xsl:if>

  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="return" mode="decl_cc_rtn">
    <xsl:call-template name="decl_cc_rtn"/>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="return" mode="decl_cc_rtn_pre">
    <xsl:call-template name="decl_cc_rtn_pre"/>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="arg" mode="decl_cc_arg">

    <xsl:call-template name="decl_cc_arg"/>
    <xsl:text> </xsl:text>
    <xsl:value-of select="@name"/>

    <xsl:if test="position() != last()">
      <xsl:text>, </xsl:text>
    </xsl:if>

  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="return" mode="decl_call_cf_rtn">
    <xsl:call-template name="decl_call_cf_rtn"/>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="return" mode="call_cf_rtn">
    <xsl:call-template name="call_cf_rtn"/>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="return" mode="call_cf_rtn_pre">
    <xsl:call-template name="call_cf_rtn_pre"/>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="arg" mode="call_cf">

    <xsl:call-template name="call_cf_arg_pre"/>
    <xsl:value-of select="@name"/>
    <xsl:call-template name="call_cf_arg_post"/>

    <xsl:if test="position() != last()">
      <xsl:text>, </xsl:text>
    </xsl:if>

  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="arg" mode="call_cf_arg_str_post">
    <xsl:variable name="arg_str_post">
      <xsl:call-template name="call_cf_arg_str_post"/>
    </xsl:variable>

    <xsl:if test="$arg_str_post != ''">
      <xsl:value-of select="concat(', ', @name, $arg_str_post)"/>
    </xsl:if>

  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="return" mode="call_cf_rtn_post">
    <xsl:call-template name="call_cf_rtn_post"/>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="return" mode="assign_call_cf_rtn">
    <xsl:call-template name="assign_call_cf_rtn"/>
  </xsl:template>


  <!--
    - openFile: print file header information
    -->
  <xsl:template name="openFile">

    <xsl:param name="file" select="'unknown_file'"/>
    <xsl:text>//</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:text>// chasm_call.cc - this file is automatically generated</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:text>//</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="$newline"/>
    <xsl:text>#include &quot;chasm_call.h&quot;</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="$newline"/>

  </xsl:template>


  <!--
    - closeFile: finish up
    -->
  <xsl:template name="closeFile">
    <xsl:value-of select="$newline"/>
  </xsl:template>

  <!--
    - openExternC: create extern "C" declaration
    -->
  <xsl:template name="openExternC">

    <xsl:text>extern "C" {</xsl:text>
    <xsl:value-of select="$newline"/>

  </xsl:template>


  <!--
    - closeExternC: close extern "C" declaration
    -->
  <xsl:template name="closeExternC">

    <xsl:text>};</xsl:text>
    <xsl:value-of select="$newline"/>

  </xsl:template>

</xsl:stylesheet>
