<!-- LANL:license
 ...........................................................................
 - This SOFTWARE has been authored by an employee or employees of the
 - University of California, operator of the Los Alamos National Laboratory
 - under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 - The U.S. Government has rights to use, reproduce, and distribute this
 - SOFTWARE.  The public may copy, distribute, prepare derivative works and
 - publicly display this SOFTWARE without charge, provided that this Notice
 - and any statement of authorship are reproduced on all copies.  Neither
 - the Government nor the University makes any warranty, express or implied,
 - or assumes any liability or responsibility for the use of this SOFTWARE.
 - If SOFTWARE is modified to produce derivative works, such modified
 - SOFTWARE should be clearly marked, so as not to confuse it with the
 - version available from LANL.
 ...........................................................................
 - LANL:license
 ...........................................................................
 -->

<!--
  - chasm_call.h.xsl: creates a header file for C functions that call
  -   Fortran module procedures.
  -
  - Output should be directed to the file, MODULE_call.h
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

  <xsl:import href="type-conversions.xsl"/>
  <xsl:import href="name-conversions.xsl"/>

  <xsl:output method="text"/>

  <xsl:variable name="newline">
<xsl:text>
</xsl:text>
  </xsl:variable>


 <!--
    - root level
    -->
  <xsl:template match="/">
    <xsl:apply-templates select="library"/>
  </xsl:template>


  <!--
    - library level
    -->
  <xsl:template match="library">

    <xsl:call-template name="openFile"/>
    <xsl:apply-templates select="scope" mode="defineF90Symbols"/>
    <xsl:apply-templates select="scope" mode="declareModuleInit"/>
    <xsl:apply-templates select="scope" mode="declareModuleNamespace"/>
    <xsl:call-template name="closeFile"/>

  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="scope" mode="defineF90Symbols">
    <xsl:if test="@kind = 'module'">
      <xsl:text>#if defined(F90_SYM_CASE_LOWER)</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:call-template name="define-init-F90-symbol-lower"/>
      <xsl:text>#endif /* F90_SYM_CASE_LOWER) */</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:value-of select="$newline"/>

      <xsl:text>#if defined(F90_SYM_CASE_MIXED)</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:call-template name="define-init-F90-symbol-mixed"/>
      <xsl:text>#endif /* F90_SYM_CASE_MIXED) */</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:value-of select="$newline"/>

      <xsl:text>#if defined(F90_SYM_CASE_UPPER)</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:call-template name="define-init-F90-symbol-upper"/>
      <xsl:text>#endif /* F90_SYM_CASE_UPPER) */</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:value-of select="$newline"/>
    </xsl:if>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="scope" mode="declareModuleInit">

    <xsl:if test="@kind = 'module'">
      <xsl:text>//</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:text>// Fortran module initialization routine</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:text>//</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:call-template name="openExternC"/>
      <xsl:call-template name="declare-init-module"/>
      <xsl:call-template name="closeExternC"/>
    </xsl:if>

  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="scope" mode="declareModuleNamespace">

    <xsl:text>//</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:text>// Namespace for Fortran module</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:text>//</xsl:text>
    <xsl:value-of select="$newline"/>

    <xsl:value-of select="$newline"/>
    <xsl:call-template name="openNamespace">
      <xsl:with-param name="module" select="@name"/>
    </xsl:call-template>

    <xsl:apply-templates select="structure" mode="define-struct"/>

    <xsl:text>  //</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:text>  // C function prototypes</xsl:text>
    <xsl:text> bridging to Fortran procedures</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:text>  //</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="$newline"/>
    <xsl:apply-templates select="method" mode="declare-CtoF-calls"/>

    <xsl:call-template name="closeNamespace">
      <xsl:with-param name="module" select="@name"/>
    </xsl:call-template>

  </xsl:template>


  <!--
    - 
    -->
  <xsl:template name="define-init-F90-symbol-lower">
      <xsl:variable name="procedure" select="concat('CHASM_INIT_', @name)"/>
      <xsl:call-template name="define-F90-symbol-lower">
        <xsl:with-param name="symbol" select="$procedure"/>
      </xsl:call-template>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template name="define-init-F90-symbol-mixed">
      <xsl:variable name="procedure" select="concat('CHASM_INIT_', @name)"/>
      <xsl:call-template name="define-F90-symbol-mixed">
        <xsl:with-param name="symbol" select="$procedure"/>
      </xsl:call-template>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template name="define-init-F90-symbol-upper">
      <xsl:variable name="procedure" select="concat('CHASM_INIT_', @name)"/>
      <xsl:call-template name="define-F90-symbol-upper">
        <xsl:with-param name="symbol" select="$procedure"/>
      </xsl:call-template>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template name="declare-init-module">

    <xsl:if test="@kind = 'module'">
      <xsl:variable name="procedure" select="concat('CHASM_INIT_', @name)"/>
      <xsl:text>  void </xsl:text>
      <xsl:call-template name="CSymbolName">
        <xsl:with-param name="symbol" select="$procedure"/>
      </xsl:call-template>
      <xsl:text>();</xsl:text>
      <xsl:value-of select="$newline"/>
    </xsl:if>

  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="structure" mode="define-struct">

    <xsl:if test="@kind = 'ftype'">
      <xsl:text>  //</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:text>  // C structs shadowing Fortrans user types</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:text>  //</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:value-of select="$newline"/>

      <xsl:text>  typedef struct {</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:apply-templates select="field" mode="define-struct"/>
      <xsl:text>  } </xsl:text>
      <xsl:value-of select="@name"/>
      <xsl:text>;</xsl:text>
      <xsl:value-of select="$newline"/>
      <xsl:value-of select="$newline"/>
    </xsl:if>

  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="method" mode="declare-CtoF-calls">
    <xsl:if test="@kind">
      <xsl:text>  </xsl:text>
      <xsl:apply-templates select="return" mode="decl_cc_rtn"/>
      <xsl:text> </xsl:text>
      <xsl:value-of select="@name"/>
      <xsl:text>(</xsl:text>
      <xsl:apply-templates select="return" mode="decl_cc_rtn_pre"/>
      <xsl:apply-templates select="arg" mode="decl_cc_arg"/>
      <xsl:apply-templates select="return" mode="decl_cc_rtn_post"/>
      <xsl:text>);</xsl:text>
      <xsl:value-of select="$newline"/>
    </xsl:if>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="field" mode="define-struct">
    <xsl:text>    </xsl:text>
    <xsl:call-template name="declTypeConvPre-FtoC"/>
    <xsl:text> </xsl:text>

    <!-- check for fortran array pointers -->
    <xsl:if test="type/indirect/type/@kind = 'farray'">
      <xsl:text>dv_</xsl:text>
    </xsl:if>

    <xsl:value-of select="@name"/>
    <xsl:call-template name="declTypeConvPost-FtoC"/>
    <xsl:text>;</xsl:text>
    <xsl:value-of select="$newline"/>
  </xsl:template>

  <!--
    - 
    -->
  <xsl:template match="return" mode="decl_cc_rtn">
    <xsl:call-template name="decl_cc_rtn"/>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="return" mode="decl_cc_rtn_pre">
    <xsl:variable name="rtn">
      <xsl:call-template name="decl_cc_rtn_pre"/>
    </xsl:variable>

    <xsl:if test="$rtn != ''">
      <xsl:value-of select="$rtn"/>
      <xsl:if test="../arg">
        <xsl:text>, </xsl:text>
      </xsl:if>
    </xsl:if>

  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="return" mode="decl_cc_rtn_post">
    <xsl:call-template name="decl_cc_rtn_post"/>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="arg" mode="decl_cc_arg">
    <xsl:if test="position() &gt; '1'">
      <xsl:text>, </xsl:text>
    </xsl:if> 
    <xsl:call-template name="decl_cc_arg"/>
  </xsl:template>


  <!--
    - openFile: print file header information
    -->
  <xsl:template name="openFile">
    <xsl:param name="file" select="'unknown_file'"/>

    <xsl:text>//
// chasm_call.h - this file is automatically generated
//

#include &lt;F90Compiler.h&gt;
#include &lt;F90Array.h&gt;
#include &lt;string&gt;

</xsl:text>

  </xsl:template>


  <!--
    - closeFile: finish up
    -->
  <xsl:template name="closeFile"/>


  <!--
    - openExternC: print code to start extern "C"
    -->
  <xsl:template name="openExternC">

    <xsl:text>extern "C" {</xsl:text>
    <xsl:value-of select="$newline"/>

  </xsl:template>


  <!--
    - closeExternC: print code to close extern "C"
    -->
  <xsl:template name="closeExternC">

    <xsl:text>}</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="$newline"/>

  </xsl:template>


  <!--
    - openNamespace: print code to start namespace for module
    -->
  <xsl:template name="openNamespace">

    <xsl:param name="module" select="'unknown_module'"/>
    <xsl:text>namespace </xsl:text>
    <xsl:value-of select="$module"/>
    <xsl:text> {</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="$newline"/>

  </xsl:template>


  <!--
    - closeNamespace: print code to close namespace for module
    -->
  <xsl:template name="closeNamespace">

    <xsl:param name="module" select="'unknown_module'"/>
    <xsl:value-of select="$newline"/>
    <xsl:text>} // namespace </xsl:text>
    <xsl:value-of select="$module"/>
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="$newline"/>

  </xsl:template>


</xsl:stylesheet>
