<!-- LANL:license
 ...........................................................................
 - This SOFTWARE has been authored by an employee or employees of the
 - University of California, operator of the Los Alamos National Laboratory
 - under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 - The U.S. Government has rights to use, reproduce, and distribute this
 - SOFTWARE.  The public may copy, distribute, prepare derivative works and
 - publicly display this SOFTWARE without charge, provided that this Notice
 - and any statement of authorship are reproduced on all copies.  Neither
 - the Government nor the University makes any warranty, express or implied,
 - or assumes any liability or responsibility for the use of this SOFTWARE.
 - If SOFTWARE is modified to produce derivative works, such modified
 - SOFTWARE should be clearly marked, so as not to confuse it with the
 - version available from LANL.
 ...........................................................................
 - LANL:license
 ...........................................................................
 -->

<!--
  - chasm_call_init.f90.xsl: creates a Fortran file to initialize Fortran
  -   function pointers.
  -
  - Output should be directed to the file, MODULE_call_init.f90
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

  <xsl:import href="name-conversions.xsl"/>
  <xsl:import href="type-conversions.xsl"/>

  <xsl:output method="text"/>

  <xsl:variable name="newline">
<xsl:text>
</xsl:text>
  </xsl:variable>


 <!--
    - root level
    -->
  <xsl:template match="/">
    <xsl:apply-templates select="library"/>
  </xsl:template>


  <!--
    - library level
    -->
  <xsl:template match="library">
    <xsl:call-template name="openFile"/>
    <xsl:apply-templates select="scope"/>
    <xsl:call-template name="closeFile"/>
  </xsl:template>


  <!--
    - scope level
    -->
  <xsl:template match="scope">
    <xsl:if test="method">
      <xsl:if test="@kind != 'global'">
        <xsl:text>subroutine CHASM_INIT_</xsl:text>
        <xsl:call-template name="moduleAbbrev"/>
        <xsl:text>()</xsl:text>
        <xsl:value-of select="$newline"/>

        <xsl:if test="@kind = 'module'">
          <xsl:text>  use </xsl:text>
          <xsl:value-of select="@name"/>
          <xsl:value-of select="$newline"/>
          <xsl:apply-templates select="method" mode="setFunctionPointers"/>
        </xsl:if>

        <xsl:text>end subroutine CHASM_INIT_</xsl:text>
        <xsl:call-template name="moduleAbbrev"/>
        <xsl:value-of select="$newline"/>
      </xsl:if>

      <xsl:if test="@kind = 'global'">
        <xsl:value-of select="$newline"/>
        <xsl:apply-templates select="method" mode="alternateReturns"/>
      </xsl:if>
    </xsl:if>
  </xsl:template>


  <!--
    - method level
    -->

  <xsl:template match="method" mode="setFunctionPointers">
    <xsl:if test="@kind">
      <xsl:text>  call SET_</xsl:text>
      <xsl:call-template name="methodName"/>
      <xsl:text>( </xsl:text>
      <xsl:value-of select="@name"/>
      <xsl:text> )</xsl:text>
      <xsl:value-of select="$newline"/>
    </xsl:if>
  </xsl:template>


  <xsl:template match="method" mode="alternateReturns">
    <xsl:variable name="rtn_pre">
      <xsl:apply-templates select="return" mode="decl_cc_rtn_pre"/>
    </xsl:variable>

    <xsl:if test="$rtn_pre != ''">
      <xsl:value-of select="concat('subroutine F_GBL_', @name, '(chasm_rtn')"/>
      <xsl:if test="arg">
	<xsl:text>, </xsl:text>
      </xsl:if>
      <xsl:apply-templates select="arg" mode="call_ff_arg"/>
      <xsl:value-of select="concat(')', $newline)"/>

      <xsl:if test="return/type/@kind = 'fchar'">
        <xsl:value-of select="concat('  external ', @name)"/>
        <xsl:value-of select="$newline"/>
        <xsl:text>  character(len=*) :: chasm_rtn</xsl:text>
        <xsl:value-of select="$newline"/>
        <xsl:text>  character(len=</xsl:text>
        <xsl:value-of select="concat(return/type/@yclen, ') :: ', @name)"/>
        <xsl:value-of select="$newline"/>
      </xsl:if>

      <xsl:apply-templates select="arg" mode="decl_ff_arg"/>

      <xsl:value-of select="concat('  chasm_rtn = ', @name, '(')"/>
      <xsl:apply-templates select="arg" mode="call_ff_arg"/>
      <xsl:text>)</xsl:text>

      <xsl:if test="return/type/@kind = 'fchar'">
        <xsl:text> // char(0)</xsl:text>
      </xsl:if>

      <xsl:value-of select="$newline"/>
      <xsl:value-of select="concat('end subroutine F_GBL_', @name, $newline)"/>
      <xsl:value-of select="$newline"/>
    </xsl:if>
  </xsl:template>


  <!--
    - openFile: print file header information
    -->
  <xsl:template name="openFile">
    <xsl:param name="file" select="'unknown_file'"/>
    <xsl:text>!</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:text>! chasm_call_init.f90</xsl:text>
    <xsl:text> - this file is automatically generated</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:text>!</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="$newline"/>
  </xsl:template>


  <!--
    - closeFile: finish up
    -->
  <xsl:template name="closeFile">
    <xsl:value-of select="$newline"/>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="arg" mode="call_ff_arg">
    <xsl:value-of select="@name"/>
    <xsl:if test="position() != last()">
      <xsl:text>, </xsl:text>
    </xsl:if>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="arg" mode="decl_ff_arg">
    <xsl:text>  </xsl:text>
    <xsl:call-template name="decl_ff_arg"/>
    <xsl:value-of select="$newline"/>
  </xsl:template>


</xsl:stylesheet>
