<!-- LANL:license
 ...........................................................................
 - This SOFTWARE has been authored by an employee or employees of the
 - University of California, operator of the Los Alamos National Laboratory
 - under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 - The U.S. Government has rights to use, reproduce, and distribute this
 - SOFTWARE.  The public may copy, distribute, prepare derivative works and
 - publicly display this SOFTWARE without charge, provided that this Notice
 - and any statement of authorship are reproduced on all copies.  Neither
 - the Government nor the University makes any warranty, express or implied,
 - or assumes any liability or responsibility for the use of this SOFTWARE.
 - If SOFTWARE is modified to produce derivative works, such modified
 - SOFTWARE should be clearly marked, so as not to confuse it with the
 - version available from LANL.
 ...........................................................................
 - LANL:license
 ...........................................................................
 -->

<!--
  - Templates for name conversions
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

  <xsl:output method="text"/>


  <!--
    - portName: name of the port for this module (FIXME)
    -    assumed to be called at a scope node
    -->
  <xsl:template name="portName">
    <xsl:value-of select="@name"/>
    <xsl:text>Port</xsl:text>
  </xsl:template>


  <!--
    - componentName: name of the component for this module (FIXME)
    -    assumed to be called at a scope node
    -->
  <xsl:template name="componentName">
    <xsl:text>ccaports.</xsl:text>
    <xsl:value-of select="@name"/>
  </xsl:template>


  <!--
    - moduleAbbrev: abbreviation for module name
    -    assumed to be called at a scope node
    -->
  <xsl:template name="moduleAbbrev">
    <xsl:choose>
      <xsl:when test="@abbrev">
        <xsl:value-of select="@abbrev"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="@name"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <!--
    - methodName: mangle method name, e.g., Module_Method
    -    assumed to be called at a method node
    -->
  <xsl:template name="methodName">
    <xsl:choose>
      <xsl:when test="../@abbrev">
        <xsl:value-of select="../@abbrev"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="../@name"/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>_</xsl:text>
    <xsl:value-of select="@name"/>
  </xsl:template>


  <!--
    - CSymbolName: mangle name to a C symbol - compiler dependent
    -->
  <xsl:template name="CSymbolName">
    <xsl:param name="symbol" select="unknown_symbol"/>
    <xsl:value-of select="$symbol"/>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template name="define-F90-symbol-lower">
    <xsl:param name="symbol" select="unknown_symbol"/>
    <xsl:text>#  define </xsl:text>
    <xsl:value-of select="$symbol"/>
    <xsl:text> F90_SYMBOL( </xsl:text>
    <xsl:call-template name="lower-case">
      <xsl:with-param name="symbol" select="$symbol"/>
    </xsl:call-template>
    <xsl:text> )</xsl:text>
    <xsl:value-of select="$newline"/>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template name="define-F90-symbol-mixed">
    <xsl:param name="symbol" select="unknown_symbol"/>
    <xsl:text>#  define </xsl:text>
    <xsl:value-of select="$symbol"/>
    <xsl:text> F90_SYMBOL( </xsl:text>
    <xsl:value-of select="$symbol"/>
    <xsl:text> )</xsl:text>
    <xsl:value-of select="$newline"/>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template name="define-F90-symbol-upper">
    <xsl:param name="symbol" select="unknown_symbol"/>
    <xsl:text>#  define </xsl:text>
    <xsl:value-of select="$symbol"/>
    <xsl:text> F90_SYMBOL( </xsl:text>
    <xsl:call-template name="upper-case">
      <xsl:with-param name="symbol" select="$symbol"/>
    </xsl:call-template>
    <xsl:text> )</xsl:text>
    <xsl:value-of select="$newline"/>
  </xsl:template>


  <!--
    - lower-case
    -->
  <xsl:template name="lower-case">
    <xsl:param name="symbol" select="unknown_symbol"/>
    <xsl:value-of select="translate($symbol,
                             'ABCDEFGHIJKLMNOPQRSTUVWXYZ',
                             'abcdefghijklmnopqrstuvwxyz')"/>
  </xsl:template>


  <!--
    - upper-case
    -->
  <xsl:template name="upper-case">
    <xsl:param name="symbol" select="unknown_symbol"/>
    <xsl:value-of select="translate($symbol,
                             'abcdefghijklmnopqrstuvwxyz',
                             'ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
  </xsl:template>


</xsl:stylesheet>
