<!-- LANL:license
 ...........................................................................
 - This SOFTWARE has been authored by an employee or employees of the
 - University of California, operator of the Los Alamos National Laboratory
 - under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 - The U.S. Government has rights to use, reproduce, and distribute this
 - SOFTWARE.  The public may copy, distribute, prepare derivative works and
 - publicly display this SOFTWARE without charge, provided that this Notice
 - and any statement of authorship are reproduced on all copies.  Neither
 - the Government nor the University makes any warranty, express or implied,
 - or assumes any liability or responsibility for the use of this SOFTWARE.
 - If SOFTWARE is modified to produce derivative works, such modified
 - SOFTWARE should be clearly marked, so as not to confuse it with the
 - version available from LANL.
 ...........................................................................
 - LANL:license
 ...........................................................................
 -->

<!--
  - Templates for type conversions
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

  <xsl:output method="text"/>

  <xsl:template name="declTypeConvPre-FtoC">
    <xsl:choose>
      <xsl:when test="type/@kind = 'fint'">
        <xsl:choose>
          <xsl:when test="type/@ikind = 'char'">
            <xsl:text>char</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'uchar'">
            <xsl:text>unsigned char</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'short'">
            <xsl:text>short</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'ushort'">
            <xsl:text>unsigned short</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'int'">
            <xsl:text>int</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'uint'">
            <xsl:text>unsigned int</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'long'">
            <xsl:text>long</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'ulong'">
            <xsl:text>unsigned long</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'longlong'">
            <xsl:text>long long</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'ulonglong'">
            <xsl:text>unsigned long long</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>UNSUPPORTED</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="type/@kind = 'ffloat'">
        <xsl:choose>
          <xsl:when test="type/@fkind = 'float'">
            <xsl:text>float</xsl:text>
          </xsl:when>
          <xsl:when test="type/@fkind = 'dbl'">
            <xsl:text>double</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>UNSUPPORTED</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="type/@kind = 'flogic'"/>
      <xsl:when test="type/@kind = 'fchar'"/>
      <xsl:when test="type/@kind = 'farray'">
        <xsl:for-each select="type/array">
          <xsl:call-template name="declTypeConvPre-FtoC"/>
        </xsl:for-each>
      </xsl:when>
      <xsl:when test="type/@kind = 'usertype'">
        <xsl:value-of select="type/@usertype"/>
      </xsl:when>
      <xsl:when test="type/@kind = 'fptr'">
        <xsl:if test="type/indirect/type/@kind = 'farray'">
          <xsl:text>F90ArrayDesc_</xsl:text>
          <xsl:value-of select="type/indirect/type/array/@rank"/>
          <xsl:text>D</xsl:text>
        </xsl:if>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>UNSUPPORTED</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="declTypeConvPost-FtoC">
    <xsl:choose>
      <xsl:when test="type/@kind = 'fint'"/>
      <xsl:when test="type/@kind = 'ffloat'"/>
      <xsl:when test="type/@kind = 'flogic'"/>
      <xsl:when test="type/@kind = 'fchar'"/>
      <xsl:when test="type/@kind = 'farray'">
        <xsl:for-each select="type/array">
          <xsl:for-each select="dimension">
            <xsl:text>[</xsl:text>
            <xsl:value-of select="@extent"/>
            <xsl:text>]</xsl:text>
          </xsl:for-each>
        </xsl:for-each>
      </xsl:when>
      <xsl:when test="type/@kind = 'usertype'"/>
      <xsl:when test="type/@kind = 'fptr'"/>
      <xsl:otherwise>
        <xsl:text>UNSUPPORTED</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <!--
    - declare cc (C to C) templates
    -   used in chasm_call.h and chasm_call.cc
    -->

  <xsl:template name="decl_cc_rtn">
    <xsl:choose>
      <xsl:when test="type/@kind = 'fvoid'">
        <xsl:text>void</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'fint'">
        <xsl:choose>
          <xsl:when test="type/@ikind = 'char'">
            <xsl:text>char</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'uchar'">
            <xsl:text>unsigned char</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'short'">
            <xsl:text>short</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'ushort'">
            <xsl:text>unsigned short</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'int'">
            <xsl:text>int</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'uint'">
            <xsl:text>unsigned int</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'long'">
            <xsl:text>long</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'ulong'">
            <xsl:text>unsigned long</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'longlong'">
            <xsl:text>long long</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'ulonglong'">
            <xsl:text>unsigned long long</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>UNSUPPORTED</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="type/@kind = 'ffloat'">
        <xsl:choose>
          <xsl:when test="type/@fkind = 'float'">
            <xsl:text>float</xsl:text>
          </xsl:when>
          <xsl:when test="type/@fkind = 'dbl'">
            <xsl:text>double</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>UNSUPPORTED</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="type/@kind = 'flogic'">
        <xsl:text>bool</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'fchar'">
        <xsl:text>void</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'farray'">
        <xsl:text>void</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'usertype'"/>
      <xsl:when test="type/@kind = 'fptr'">
        <xsl:text>void</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>UNSUPPORTED</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <xsl:template name="decl_cc_rtn_pre">
    <xsl:choose>
      <xsl:when test="type/@kind = 'fvoid'"/>
      <xsl:when test="type/@kind = 'fint'"/>
      <xsl:when test="type/@kind = 'ffloat'"/>
      <xsl:when test="type/@kind = 'flogic'"/>
      <xsl:when test="type/@kind = 'fchar'">
        <xsl:text>std::string&amp;</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'farray'">
        <xsl:text>F90::Descriptor&amp;</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'usertype'"/>
      <xsl:when test="type/@kind = 'fptr'">
        <xsl:text>F90::Descriptor&amp;</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>UNSUPPORTED</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <xsl:template name="decl_cc_rtn_post">
    <xsl:choose>
      <xsl:when test="type/@kind = 'fvoid'"/>
      <xsl:when test="type/@kind = 'fint'"/>
      <xsl:when test="type/@kind = 'ffloat'"/>
      <xsl:when test="type/@kind = 'flogic'"/>
      <xsl:when test="type/@kind = 'fchar'"/>
      <xsl:when test="type/@kind = 'farray'"/>
      <xsl:when test="type/@kind = 'usertype'"/>
      <xsl:when test="type/@kind = 'fptr'"/>
      <xsl:otherwise>
        <xsl:text>UNSUPPORTED</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <!--
    - declare cf (C to F) templates
    -   used in chasm_call.cc
    -->

  <xsl:template name="decl_cf_arg">
    <xsl:choose>
      <xsl:when test="type/@kind = 'fint'">
        <xsl:choose>
          <xsl:when test="type/@ikind = 'char'">
            <xsl:text>char*</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'uchar'">
            <xsl:text>unsigned char*</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'short'">
            <xsl:text>short*</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'ushort'">
            <xsl:text>unsigned short*</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'int'">
            <xsl:text>int*</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'uint'">
            <xsl:text>unsigned int*</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'long'">
            <xsl:text>long*</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'ulong'">
            <xsl:text>unsigned long*</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'longlong'">
            <xsl:text>long long*</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'ulonglong'">
            <xsl:text>unsigned long long*</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>UNSUPPORTED</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="type/@kind = 'ffloat'">
        <xsl:choose>
          <xsl:when test="type/@fkind = 'float'">
            <xsl:text>float*</xsl:text>
          </xsl:when>
          <xsl:when test="type/@fkind = 'dbl'">
            <xsl:text>double*</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>UNSUPPORTED*</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="type/@kind = 'flogic'"/>
      <xsl:when test="type/@kind = 'fchar'">
        <xsl:if test="@intent = 'in'">
          <xsl:text>const </xsl:text>
        </xsl:if>
        <xsl:text>char*</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'farray'">
        <xsl:text>void*</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'usertype'">
        <xsl:value-of select="../../@name"/>
        <xsl:text>::</xsl:text>
        <xsl:value-of select="type/@usertype"/>
        <xsl:text>*</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'fptr'">
        <xsl:for-each select="type/indirect">
          <xsl:call-template name="decl_cf_arg"/>
        </xsl:for-each>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>UNSUPPORTED*</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <!--
    - Declare (in C) return type conversions of a fortran procedure
    - when called from C
    -->
  <xsl:template name="decl_cf_rtn">
    <xsl:choose>
      <xsl:when test="type/@kind = 'fvoid'">
        <xsl:text>void</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'fint'">
        <xsl:choose>
          <xsl:when test="type/@ikind = 'char'">
            <xsl:text>char</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'uchar'">
            <xsl:text>unsigned char</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'short'">
            <xsl:text>short</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'ushort'">
            <xsl:text>unsigned short</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'int'">
            <xsl:text>int</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'uint'">
            <xsl:text>unsigned int</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'long'">
            <xsl:text>long</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'ulong'">
            <xsl:text>unsigned long</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'longlong'">
            <xsl:text>long long</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'ulonglong'">
            <xsl:text>unsigned long long</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>UNSUPPORTED</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="type/@kind = 'ffloat'">
        <xsl:choose>
          <xsl:when test="type/@fkind = 'float'">
            <xsl:text>float</xsl:text>
          </xsl:when>
          <xsl:when test="type/@fkind = 'dbl'">
            <xsl:text>double</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>UNSUPPORTED</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="type/@kind = 'flogic'">
        <xsl:text>long</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'fchar'">
        <xsl:text>void</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'farray'">
        <xsl:text>void</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'usertype'"/>
      <xsl:when test="type/@kind = 'fptr'">
        <xsl:text>void</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>UNSUPPORTED</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>



  <xsl:template name="decl_cf_rtn_pre">
    <xsl:choose>
      <xsl:when test="type/@kind = 'fvoid'"/>
      <xsl:when test="type/@kind = 'fint'"/>
      <xsl:when test="type/@kind = 'ffloat'"/>
      <xsl:when test="type/@kind = 'flogic'"/>
      <xsl:when test="type/@kind = 'fchar'">
        <xsl:text>char*</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'farray'">
        <xsl:text>void*</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'usertype'"/>
      <xsl:when test="type/@kind = 'fptr'">
        <xsl:text>void*</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>UNSUPPORTED</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>



  <xsl:template name="decl_cf_rtn_post">
    <xsl:choose>
      <xsl:when test="type/@kind = 'fvoid'"/>
      <xsl:when test="type/@kind = 'fint'"/>
      <xsl:when test="type/@kind = 'ffloat'"/>
      <xsl:when test="type/@kind = 'flogic'"/>
      <xsl:when test="type/@kind = 'fchar'">
        <xsl:text>, int</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'farray'"/>
      <xsl:when test="type/@kind = 'usertype'"/>
      <xsl:when test="type/@kind = 'fptr'"/>
      <xsl:otherwise>
        <xsl:text>UNSUPPORTED</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>



  <xsl:template name="decl_call_cf_rtn">
    <xsl:choose>
      <xsl:when test="type/@kind = 'fvoid'"/>
      <xsl:when test="type/@kind = 'fint'"/>
      <xsl:when test="type/@kind = 'ffloat'"/>
      <xsl:when test="type/@kind = 'flogic'"/>
      <xsl:when test="type/@kind = 'fchar'">
        <xsl:text>  char char_rtn[</xsl:text>
        <xsl:value-of select="1+type/@clen"/>
        <xsl:text>];</xsl:text>
        <xsl:value-of select="$newline"/>
      </xsl:when>
      <xsl:when test="type/@kind = 'farray'"/>
      <xsl:when test="type/@kind = 'usertype'"/>
      <xsl:when test="type/@kind = 'fptr'"/>
      <xsl:otherwise>
        <xsl:text>UNSUPPORTED</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>



  <xsl:template name="call_cf_rtn">
    <xsl:choose>
      <xsl:when test="type/@kind = 'fvoid'"/>
      <xsl:when test="type/@kind = 'fchar'"/>
      <xsl:when test="type/@kind = 'farray'"/>
      <xsl:when test="type/@kind = 'usertype'"/>
      <xsl:when test="type/@kind = 'fptr'"/>
      <xsl:otherwise>
        <xsl:text>return </xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <xsl:template name="call_cf_rtn_pre">
    <xsl:choose>
      <xsl:when test="type/@kind = 'fchar'">
        <xsl:text>char_rtn</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'farray'">
        <xsl:text>chasm_rtn.descriptor()</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'fptr'">
        <xsl:text>chasm_rtn.descriptor()</xsl:text>
      </xsl:when>
    </xsl:choose>
  </xsl:template>


  <xsl:template name="call_cf_rtn_post">
    <xsl:choose>
      <xsl:when test="type/@kind = 'fchar'">
        <xsl:value-of select="concat(', ', 1+type/@clen)"/>
      </xsl:when>
    </xsl:choose>
  </xsl:template>


  <xsl:template name="assign_call_cf_rtn">
    <xsl:choose>
      <xsl:when test="type/@kind = 'fvoid'"/>
      <xsl:when test="type/@kind = 'fint'"/>
      <xsl:when test="type/@kind = 'ffloat'"/>
      <xsl:when test="type/@kind = 'flogic'"/>
      <xsl:when test="type/@kind = 'fchar'">
        <xsl:text>  chasm_rtn = char_rtn;</xsl:text>
        <xsl:value-of select="$newline"/>
      </xsl:when>
      <xsl:when test="type/@kind = 'farray'"/>
      <xsl:when test="type/@kind = 'usertype'"/>
      <xsl:when test="type/@kind = 'fptr'"/>
      <xsl:otherwise>
        <xsl:text>UNSUPPORTED</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>



  <xsl:template name="decl_cc_arg">
    <xsl:choose>
      <xsl:when test="type/@kind = 'fint'">
        <xsl:choose>
          <xsl:when test="type/@ikind = 'char'">
            <xsl:text>char&amp;</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'uchar'">
            <xsl:text>unsigned char&amp;</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'short'">
            <xsl:text>short&amp;</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'ushort'">
            <xsl:text>unsigned short&amp;</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'int'">
            <xsl:text>int&amp;</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'uint'">
            <xsl:text>unsigned int&amp;</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'long'">
            <xsl:text>long&amp;</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'ulong'">
            <xsl:text>unsigned long&amp;</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'longlong'">
            <xsl:text>long long&amp;</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'ulonglong'">
            <xsl:text>unsigned long long&amp;</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>UNSUPPORTED</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="type/@kind = 'ffloat'">
        <xsl:choose>
          <xsl:when test="type/@fkind = 'float'">
            <xsl:text>float&amp;</xsl:text>
          </xsl:when>
          <xsl:when test="type/@fkind = 'dbl'">
            <xsl:text>double&amp;</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>UNSUPPORTED&amp;</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="type/@kind = 'flogic'"/>
      <xsl:when test="type/@kind = 'fchar'">
        <xsl:text>std::string&amp;</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'farray'">
        <xsl:text>F90::Descriptor&amp;</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'usertype'">
        <xsl:value-of select="../../@name"/>
        <xsl:text>::</xsl:text>
        <xsl:value-of select="type/@usertype"/>
        <xsl:text>&amp;</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'fptr'">
        <xsl:for-each select="type/indirect">
          <xsl:call-template name="decl_cc_arg"/>
        </xsl:for-each>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>UNSUPPORTED&amp;</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <xsl:template name="call_cf_arg_pre">
    <xsl:choose>
      <xsl:when test="type/@kind = 'fint'">
        <xsl:choose>
          <xsl:when test="type/@ikind = 'char'">
            <xsl:text>&amp;</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'uchar'">
            <xsl:text>&amp;</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'short'">
            <xsl:text>&amp;</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'ushort'">
            <xsl:text>&amp;</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'int'">
            <xsl:text>&amp;</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'uint'">
            <xsl:text>&amp;</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'long'">
            <xsl:text>&amp;</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'ulong'">
            <xsl:text>&amp;</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'longlong'">
            <xsl:text>&amp;</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'ulonglong'">
            <xsl:text>&amp;</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>UNSUPPORTED</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="type/@kind = 'ffloat'">
        <xsl:choose>
          <xsl:when test="type/@fkind = 'float'">
            <xsl:text>&amp;</xsl:text>
          </xsl:when>
          <xsl:when test="type/@fkind = 'dbl'">
            <xsl:text>&amp;</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>&amp;UNSUPPORTED</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="type/@kind = 'flogic'"/>
      <xsl:when test="type/@kind = 'fchar'"/>
      <xsl:when test="type/@kind = 'farray'"/>
      <xsl:when test="type/@kind = 'usertype'">
        <xsl:text>&amp;</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'fptr'"/>
      <xsl:otherwise>
        <xsl:text>&amp;UNSUPPORTED</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <xsl:template name="call_cf_arg_post">
    <xsl:choose>
      <xsl:when test="type/@kind = 'fint'"/>
      <xsl:when test="type/@kind = 'ffloat'"/>
      <xsl:when test="type/@kind = 'flogic'"/>
      <xsl:when test="type/@kind = 'fchar'">
        <xsl:text>.c_str()</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'farray'">
        <xsl:text>.descriptor()</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'usertype'"/>
      <xsl:when test="type/@kind = 'fptr'">
        <xsl:for-each select="type/indirect">
          <xsl:call-template name="call_cf_arg_post"/>
        </xsl:for-each>
      </xsl:when>
      <xsl:otherwise/>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="call_cf_arg_str_post">
    <xsl:choose>
      <xsl:when test="type/@kind = 'fint'"/>
      <xsl:when test="type/@kind = 'ffloat'"/>
      <xsl:when test="type/@kind = 'flogic'"/>
      <xsl:when test="type/@kind = 'fchar'">
        <xsl:text>.length()</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'farray'"/>
      <xsl:when test="type/@kind = 'usertype'"/>
      <xsl:when test="type/@kind = 'fptr'"/>
      <xsl:otherwise/>
    </xsl:choose>
  </xsl:template>


  <xsl:template name="decl_ff_arg_short">
    <xsl:choose>
      <xsl:when test="type/@kind = 'fint'">
        <xsl:text>integer</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'ffloat'">
        <xsl:text>real</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'flogic'">
        <xsl:text>logical</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>UNSUPPORTED</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="decl_ff_arg">
    <xsl:choose>
      <xsl:when test="type/@kind = 'fint'">
        <xsl:call-template name="decl_ff_arg_short"/>
        <xsl:value-of select="concat(' :: ', @name)"/>
      </xsl:when>
      <xsl:when test="type/@kind = 'ffloat'">
        <xsl:call-template name="decl_ff_arg_short"/>
        <xsl:value-of select="concat(' :: ', @name)"/>
      </xsl:when>
      <xsl:when test="type/@kind = 'flogic'">
        <xsl:call-template name="decl_ff_arg_short"/>
        <xsl:value-of select="concat(' :: ', @name)"/>
      </xsl:when>
      <xsl:when test="type/@kind = 'fchar'">
        <xsl:text>character(len=</xsl:text>
        <xsl:choose>
          <xsl:when test="type/@clen = '*'">
            <xsl:text>*</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="1+type/@clen"/>
          </xsl:otherwise>
        </xsl:choose>    
        <xsl:value-of select="concat(') :: ', @name)"/>
      </xsl:when>
      <xsl:when test="type/@kind = 'farray'">
        <xsl:for-each select="type/array">
          <xsl:call-template name="decl_ff_arg_short"/>
        </xsl:for-each>

        <xsl:text>, dimension(</xsl:text>
        <xsl:for-each select="type/array/dimension">
          <xsl:text>:</xsl:text>
          <xsl:if test="position() != last()">
            <xsl:text>,</xsl:text>
          </xsl:if>
        </xsl:for-each>
        <xsl:text>)</xsl:text>
        <xsl:value-of select="concat(' :: ', @name)"/>
      </xsl:when>
      <xsl:when test="type/@kind = 'usertype'"/>
      <xsl:when test="type/@kind = 'fptr'"/>
      <xsl:otherwise/>
    </xsl:choose>
  </xsl:template>

</xsl:stylesheet>
